@@ -23,6 +23,8 @@
 #include "unit_cell/atomic_conf.hpp"
 #include "potential/xc_functional.hpp"
 
+using namespace sddk;
+
 double const rmin{1e-5};
 
 class Free_atom : public sirius::Atom_type
@@ -677,7 +679,7 @@ void generate_atom_file(Free_atom&         a,
     atom_class.generate_radial_functions(relativity_t::none);
     pstdout pout(Communicator::self());
     atom_class.write_enu(pout);
-    pout.flush();
+    std::cout << pout.flush(0);
 
     auto lo_to_str = [](local_orbital_descriptor lod) {
         std::stringstream s;
