@@ -2,9 +2,9 @@
 
 void test1()
 {
-    sddk::HDF5_tree f("f.h5", hdf5_access_t::truncate);
+    sddk::HDF5_tree f("f.h5", sddk::hdf5_access_t::truncate);
     
-    mdarray<double, 2> dat(2, 4);
+    sddk::mdarray<double, 2> dat(2, 4);
     dat.zero();
     
     dat(0, 0) = 1.1;
@@ -28,20 +28,20 @@ void test1()
 
 void test2()
 {
-    sddk::HDF5_tree f("f.h5", hdf5_access_t::truncate);
+    sddk::HDF5_tree f("f.h5", sddk::hdf5_access_t::truncate);
     f.create_node("node1");
 
-    mdarray<double, 2> md1(2, 4);
+    sddk::mdarray<double, 2> md1(2, 4);
     md1.zero();
     f["node1"].write("md1", md1);
     f["node1"].write(0, md1);
 
-    mdarray<double_complex, 2> md2(2, 4);
+    sddk::mdarray<double_complex, 2> md2(2, 4);
     md2.zero();
     f["node1"].write("md2", md2);
     f["node1"].write(1, md2);
 
-    mdarray<int, 2> md3(2, 4);
+    sddk::mdarray<int, 2> md3(2, 4);
     md3.zero();
     f["node1"].write("md3", md3);
     f["node1"].write(2, md3);
@@ -49,18 +49,18 @@ void test2()
 
 void test3()
 {
-    sddk::HDF5_tree f("f.h5", hdf5_access_t::read_only);
+    sddk::HDF5_tree f("f.h5", sddk::hdf5_access_t::read_only);
 
-    mdarray<double, 2> md1(2, 4);
+    sddk::mdarray<double, 2> md1(2, 4);
     f["node1"].read("md1", md1);
     f["node1"].read(0, md1);
 
-    mdarray<double_complex, 2> md2(2, 4);
+    sddk::mdarray<double_complex, 2> md2(2, 4);
     md2.zero();
     f["node1"].read("md2", md2);
     f["node1"].read(1, md2);
 
-    mdarray<int, 2> md3(2, 4);
+    sddk::mdarray<int, 2> md3(2, 4);
     md3.zero();
     f["node1"].read("md3", md3);
     f["node1"].read(2, md3);
