@@ -2,26 +2,26 @@
 
 using namespace sirius;
 
-mdarray<int, 1> f1()
+sddk::mdarray<int, 1> f1()
 {
-    mdarray<int, 1> aa;
-    aa = mdarray<int, 1>(4);
+    sddk::mdarray<int, 1> aa;
+    aa = sddk::mdarray<int, 1>(4);
     for (int i = 0; i < 4; i++) aa(i) = 200 + i;
     return aa;
 }
 
 void f2()
 {
-    mdarray<int, 1> a1(4);
+    sddk::mdarray<int, 1> a1(4);
     for (int i = 0; i < 4; i++) a1(i) = 100 + i;
 
-    mdarray<int, 1> a2 = f1();
+    sddk::mdarray<int, 1> a2 = f1();
     for (int i = 0; i < 4; i++)
     {
         std::cout << "a1(" << i << ")=" << a1(i) << std::endl;
         std::cout << "a2(" << i << ")=" << a2(i) << std::endl;
     }
-    mdarray<int, 1> a3(std::move(a2));
+    sddk::mdarray<int, 1> a3(std::move(a2));
 //== 
 //== //    a1.deallocate();
 //== //
@@ -33,17 +33,17 @@ void f2()
 //== //    }
 //== //
 //== //
-//== //    mdarray<int, 1> a3 = a2;
+//== //    sddk::mdarray<int, 1> a3 = a2;
 //== //    
     for (int i = 0; i < 4; i++)
     {
         std::cout << "a3(" << i << ")=" << a3(i) << std::endl;
     }
  
-    mdarray<int, 1> a4;
+    sddk::mdarray<int, 1> a4;
     a4 = std::move(a3);
 
-    a4 = mdarray<int, 1>(20);
+    a4 = sddk::mdarray<int, 1>(20);
 }
 
 void f3()
@@ -52,7 +52,7 @@ void f3()
         #pragma omp parallel
         {
             int tid = omp_get_thread_num();
-            mdarray<double_complex, 2> a(100, 100);
+            sddk::mdarray<double_complex, 2> a(100, 100);
             a(0, 0) = double_complex(tid, tid);
         }
     }
@@ -60,20 +60,20 @@ void f3()
 
 void f4()
 {
-    mdarray<int, 1> buf;
+    sddk::mdarray<int, 1> buf;
 
-    buf = mdarray<int, 1>(100, memory_t::host, "buf");
+    buf = sddk::mdarray<int, 1>(100, sddk::memory_t::host, "buf");
 
-    buf = mdarray<int, 1>(200, memory_t::host, "buf");
+    buf = sddk::mdarray<int, 1>(200, sddk::memory_t::host, "buf");
     
-    //buf = mdarray<int, 1>(300, memory_t::host | memory_t::device, "buf");
+    //buf = sddk::mdarray<int, 1>(300, memory_t::host | memory_t::device, "buf");
 
 
 }
 
 void f5()
 {
-    mdarray<double, 3> a;
+    sddk::mdarray<double, 3> a;
 
     if (a.size(0) != 0 || a.size(1) != 0 || a.size(2) != 0) {
         printf("wrong sizes\n");
@@ -81,13 +81,13 @@ void f5()
 }
 
 template <typename T, int N>
-void f6(mdarray<T, N>& a)
+void f6(sddk::mdarray<T, N>& a)
 {
-    std::array<mdarray_index_descriptor, N> dims;
+    std::array<sddk::mdarray_index_descriptor, N> dims;
     for (int i = 0; i < N; i++) {
-        dims[i] = mdarray_index_descriptor(0, 10);
+        dims[i] = sddk::mdarray_index_descriptor(0, 10);
     }
-    a = mdarray<T, N>(dims);
+    a = sddk::mdarray<T, N>(dims);
     a[0] = 100;
     a[a.size() - 1] = 200;
 }
@@ -104,7 +104,7 @@ int main(int argn, char **argv)
 
     f5();
     
-    mdarray<double, 2> a;
+    sddk::mdarray<double, 2> a;
     f6(a);
 
     sirius::finalize();
