@@ -1,4 +1,4 @@
-#include <sirius.h>
+#include <sirius.hpp>
 
 using namespace sirius;
 
@@ -6,7 +6,7 @@ int run_test(cmd_args& args)
 {
     Simulation_context ctx;
     ctx.import(args);
-    std::cout << ctx.control().verbosity_ << "\n";
+    std::cout << ctx.verbosity() << "\n";
     return 0;
 }
 
