@@ -8,7 +8,7 @@
 
 namespace sirius {
 
-Communicator make_sirius_comm(pybind11::object py_comm)
+inline sddk::Communicator make_sirius_comm(pybind11::object py_comm)
 {
     PyObject* py_obj = py_comm.ptr();
     MPI_Comm* comm_p  = PyMPIComm_Get(py_obj);
@@ -18,12 +18,12 @@ Communicator make_sirius_comm(pybind11::object py_comm)
     int rank = -1;
     MPI_Comm_rank(*comm_p, &rank);
 
-    Communicator sirius_comm(*comm_p);
+    sddk::Communicator sirius_comm(*comm_p);
     return sirius_comm;
 }
 
 
-pybind11::handle make_pycomm(const Communicator& comm) {
+inline pybind11::handle make_pycomm(const sddk::Communicator& comm) {
     return pybind11::handle(PyMPIComm_New(comm.mpi_comm()));
 }
 
