@@ -4,7 +4,6 @@ import h5py
 from mpi4py import MPI
 from .coefficient_array import PwCoeffs, CoefficientArray
 from .py_sirius import MemoryEnum
-from .ot import matview
 from .logger import Logger
 import numpy as np
 
@@ -47,7 +46,8 @@ def load_state(filename, kset, name, dtype):
         files = glob.glob(filename)
     else:
         files = [filename]
-
+    if not len(files) > 0:
+        raise Exception('no files found: ', filename)
     for fi in files:
         with h5py.File(fi, 'r') as fh5:
             for key in fh5[name].keys():
@@ -69,6 +69,7 @@ def store_pw_coeffs(kpointset, cn, ki=None, ispn=None):
     cn     -- numpy array
     ispn   -- spin component
     """
+    from .ot import matview
 
     if isinstance(cn, PwCoeffs):
         assert (ki is None)
@@ -76,7 +77,6 @@ def store_pw_coeffs(kpointset, cn, ki=None, ispn=None):
         for key, v in cn.items():
             k, ispn = key
             n, m = v.shape
-            assert (np.isclose(matview(v).H * v, np.eye(m, m)).all())
             psi = kpointset[k].spinor_wave_functions()
             psi.pw_coeffs(ispn)[:, :v.shape[1]] = v
             on_device = psi.preferred_memory_t() == MemoryEnum.device
@@ -95,11 +95,23 @@ def DFT_ground_state_find(num_dft_iter=1, config='sirius.json'):
     run DFT_ground_state
 
     Keyword Arguments:
-    num_dft_iter -- (Default 1) number of SCF interations
+    num_dft_iter -- (Default 1) number of SCF iterations
     config       -- json configuration / or dictionary (from json)
-    """
 
-    from . import Simulation_context, K_point_set, DFT_ground_state
+    Returns:
+    A dictionary with keys
+    - E (total energy)
+    - dft_gs (DFT_ground_state instance)
+    - kpointset (K_point_set instance)
+    - hamiltonian (Hamiltonian instance)
+    - density (Density instance)
+    - potential (Potential instance)
+    - ctx (Simulation_context instance)
+    """
+    from . import (Simulation_context,
+                   K_point_set,
+                   DFT_ground_state,
+                   vector3d_double)
     import json
     if isinstance(config, dict):
         ctx = Simulation_context(json.dumps(config))
@@ -108,23 +120,24 @@ def DFT_ground_state_find(num_dft_iter=1, config='sirius.json'):
         siriusJson = json.load(open(config))
         ctx = Simulation_context(json.dumps(siriusJson))
     ctx.initialize()
-
-    if 'shiftk' in siriusJson['parameters']:
-        # make sure shiftk is not a list of floats
-        shiftk = [int(x) for x in siriusJson['parameters']['shiftk']]
+    if 'vk' in siriusJson['parameters']:
+        vk = siriusJson['parameters']['vk']
+        kPointSet = K_point_set(ctx, [vector3d_double(x) for x in vk])
     else:
-        shiftk = [0, 0, 0]
-    if 'ngridk' in siriusJson['parameters']:
-        gridk = siriusJson['parameters']['ngridk']
-    if 'use_symmetry' in siriusJson['parameters']:
-        use_symmetry = siriusJson['parameters']['use_symmetry']
-    else:
-        use_symmetry = True
-
-    kPointSet = K_point_set(ctx, gridk, shiftk, use_symmetry)
+        if 'shiftk' in siriusJson['parameters']:
+            # make sure shiftk is not a list of floats
+            shiftk = [int(x) for x in siriusJson['parameters']['shiftk']]
+        else:
+            shiftk = [0, 0, 0]
+        if 'ngridk' in siriusJson['parameters']:
+           gridk = siriusJson['parameters']['ngridk']
+        if 'use_symmetry' in siriusJson['parameters']:
+            use_symmetry = siriusJson['parameters']['use_symmetry']
+        else:
+            use_symmetry = True
+        kPointSet = K_point_set(ctx, gridk, shiftk, use_symmetry)
 
     dft_gs = DFT_ground_state(kPointSet)
-
     dft_gs.initial_state()
 
     if 'potential_tol' not in siriusJson['parameters']:
@@ -138,16 +151,13 @@ def DFT_ground_state_find(num_dft_iter=1, config='sirius.json'):
         energy_tol = siriusJson['parameters']['energy_tol']
     write_status = False
 
-    initial_tol = energy_tol
+    initial_tol = 1e-2  # TODO: magic number
     E0 = dft_gs.find(potential_tol, energy_tol, initial_tol, num_dft_iter, write_status)
-    ks = dft_gs.k_point_set()
-    hamiltonian = dft_gs.hamiltonian()
 
     return {
         'E': E0,
         'dft_gs': dft_gs,
-        'kpointset': ks,
-        'hamiltonian': hamiltonian,
+        'kpointset': kPointSet,
         'density': dft_gs.density(),
         'potential': dft_gs.potential(),
         'ctx': ctx
@@ -163,7 +173,7 @@ def dphk_factory(config='sirius.json'):
     config -- (Default sirius.json) json configuration
     """
 
-    from . import Band, K_point_set, Potential, Density, Hamiltonian, Simulation_context
+    from . import Band, K_point_set, Potential, Density, Hamiltonian, Simulation_context, vector3d_double
     import json
 
     siriusJson = json.load(open(config))
@@ -171,24 +181,27 @@ def dphk_factory(config='sirius.json'):
     ctx.initialize()
     density = Density(ctx)
     potential = Potential(ctx)
-    hamiltonian = Hamiltonian(ctx, potential)
+    hamiltonian = Hamiltonian(potential)
 
-    if 'shiftk' in siriusJson['parameters']:
-        shiftk = siriusJson['parameters']['shiftk']
+    if 'vk' in siriusJson['parameters']:
+        vk = siriusJson['parameters']['vk']
+        kPointSet = K_point_set(ctx, [vector3d_double(x) for x in vk])
     else:
-        shiftk = [0, 0, 0]
-    if 'ngridk' in siriusJson['parameters']:
-        gridk = siriusJson['parameters']['ngridk']
-    use_symmetry = siriusJson['parameters']['use_symmetry']
+        if 'shiftk' in siriusJson['parameters']:
+            shiftk = siriusJson['parameters']['shiftk']
+        else:
+            shiftk = [0, 0, 0]
+        if 'ngridk' in siriusJson['parameters']:
+            gridk = siriusJson['parameters']['ngridk']
+        use_symmetry = siriusJson['parameters']['use_symmetry']
+        kPointSet = K_point_set(ctx, gridk, shiftk, use_symmetry)
 
-    kPointSet = K_point_set(ctx, gridk, shiftk, use_symmetry)
     Band(ctx).initialize_subspace(kPointSet, hamiltonian)
 
     return {
         'kpointset': kPointSet,
         'density': density,
         'potential': potential,
-        'hamiltonian': hamiltonian
     }
 
 
@@ -223,6 +236,11 @@ def kpoint_index(kp, ctx):
     shiftk = np.array(pm.shiftk, dtype=np.int)
     ngridk = np.array(pm.ngridk, dtype=np.int)
     ik = np.array(kp.vk) * ngridk - shiftk/2
-    assert np.isclose(ik-ik.astype(np.int), 0).all()
-    ik = ik.astype(np.int)
+    if not np.isclose(ik-ik.astype(np.int), 0).all():
+        # single k-point given in vk
+        print('WARNING: could not identify k-point index')
+        ik = [0, 0, 0]
+    else:
+        ik = ik.astype(np.int)
+
     return tuple(ik)
