@@ -25,7 +25,8 @@
 #ifndef __SERIALIZER_HPP__
 #define __SERIALIZER_HPP__
 
-#include "communicator.hpp"
+#include <limits>
+#include "mpi/communicator.hpp"
 
 namespace sddk {
 
