@@ -35,7 +35,7 @@ class Beta_projectors : public Beta_projectors_base<T>
 {
   protected:
     bool prepared_{false};
-    matrix<std::complex<T>> beta_pw_all_atoms_;
+    sddk::matrix<std::complex<T>> beta_pw_all_atoms_;
     /// Generate plane-wave coefficients for beta-projectors of atom types.
     void generate_pw_coefs_t()
     {
@@ -57,7 +57,7 @@ class Beta_projectors : public Beta_projectors_base<T>
         #pragma omp parallel for
         for (int igkloc = 0; igkloc < this->num_gkvec_loc(); igkloc++) {
             /* vs = {r, theta, phi} */
-            auto vs = SHT::spherical_coordinates(this->gkvec_.template gkvec_cart<index_domain_t::local>(igkloc));
+            auto vs = SHT::spherical_coordinates(this->gkvec_.template gkvec_cart<sddk::index_domain_t::local>(igkloc));
             /* compute real spherical harmonics for G+k vector */
             std::vector<double> gkvec_rlm(utils::lmmax(this->ctx_.unit_cell().lmax()));
             sf::spherical_harmonics(this->ctx_.unit_cell().lmax(), vs[1], vs[2], &gkvec_rlm[0]);
@@ -70,7 +70,8 @@ class Beta_projectors : public Beta_projectors_base<T>
                     int lm    = atom_type.indexb(xi).lm;
                     int idxrf = atom_type.indexb(xi).idxrf;
 
-                    this->pw_coeffs_t_(igkloc, atom_type.offset_lo() + xi, 0) = static_cast<std::complex<T>>(z[l] * gkvec_rlm[lm] * ri_val(idxrf));
+                    this->pw_coeffs_t_(igkloc, atom_type.offset_lo() + xi, 0) =
+                        static_cast<std::complex<T>>(z[l] * gkvec_rlm[lm] * ri_val(idxrf));
                 }
             }
         }
@@ -79,14 +80,13 @@ class Beta_projectors : public Beta_projectors_base<T>
             auto c1 = this->pw_coeffs_t_.checksum();
             comm.allreduce(&c1, 1);
             if (comm.rank() == 0) {
-                utils::print_checksum("beta_pw_coeffs_t", c1);
+                utils::print_checksum("beta_pw_coeffs_t", c1, std::cout);
             }
         }
-
     }
 
   public:
-    Beta_projectors(Simulation_context& ctx__, Gvec const& gkvec__)
+    Beta_projectors(Simulation_context& ctx__, sddk::Gvec const& gkvec__)
         : Beta_projectors_base<T>(ctx__, gkvec__, 1)
     {
         PROFILE("sirius::Beta_projectors");
@@ -98,19 +98,20 @@ class Beta_projectors : public Beta_projectors_base<T>
         /* special treatment for beta-projectors as they are mostly often used */
         switch (this->ctx_.processing_unit()) {
             /* beta projectors for atom types will be stored on GPU for the entire run */
-            case device_t::GPU: {
+            case sddk::device_t::GPU: {
                 this->reallocate_pw_coeffs_t_on_gpu_ = false;
-                this->pw_coeffs_t_.allocate(memory_t::device).copy_to(memory_t::device);
+                this->pw_coeffs_t_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
                 break;
             }
             /* generate beta projectors for all atoms */
-            case device_t::CPU: {
-                beta_pw_all_atoms_ = matrix<std::complex<T>>(this->num_gkvec_loc(), this->ctx_.unit_cell().mt_lo_basis_size());
+            case sddk::device_t::CPU: {
+                beta_pw_all_atoms_ = sddk::matrix<std::complex<T>>(this->num_gkvec_loc(),
+                            this->ctx_.unit_cell().mt_lo_basis_size());
                 for (int ichunk = 0; ichunk < this->num_chunks(); ichunk++) {
                     /* wrap the the pointer in the big array beta_pw_all_atoms */
-                    this->pw_coeffs_a_ = matrix<std::complex<T>>(&beta_pw_all_atoms_(0, this->chunk(ichunk).offset_),
+                    this->pw_coeffs_a_ = sddk::matrix<std::complex<T>>(&beta_pw_all_atoms_(0, this->chunk(ichunk).offset_),
                                                                 this->num_gkvec_loc(), this->chunk(ichunk).num_beta_);
-                    Beta_projectors_base<T>::generate(ichunk, 0);
+                    Beta_projectors_base<T>::generate(sddk::memory_t::host, ichunk, 0);
                 }
                 break;
             }
@@ -123,11 +124,13 @@ class Beta_projectors : public Beta_projectors_base<T>
             TERMINATE("beta projectors are already prepared");
         }
         switch (this->ctx_.processing_unit()) {
-            case device_t::GPU: {
+            case sddk::device_t::GPU: {
                 Beta_projectors_base<T>::prepare();
                 break;
             }
-            case device_t::CPU: break;
+            case sddk::device_t::CPU: {
+                break;
+            }
         }
         prepared_ = true;
     }
@@ -138,27 +141,25 @@ class Beta_projectors : public Beta_projectors_base<T>
             TERMINATE("beta projectors are already dismissed");
         }
         switch (this->ctx_.processing_unit()) {
-            case device_t::GPU: {
+            case sddk::device_t::GPU: {
                 Beta_projectors_base<T>::dismiss();
                 break;
             }
-            case device_t::CPU: break;
+            case sddk::device_t::CPU: {
+                break;
+            }
         }
         prepared_ = false;
     }
 
-    void generate(int chunk__)
+    void generate(sddk::memory_t mem__, int chunk__)
     {
-        switch (this->ctx_.processing_unit()) {
-            case device_t::CPU: {
-                this->pw_coeffs_a_ = matrix<std::complex<T>>(&beta_pw_all_atoms_(0, this->chunk(chunk__).offset_),
+        if (is_host_memory(mem__)) {
+            this->pw_coeffs_a_ = sddk::matrix<std::complex<T>>(&beta_pw_all_atoms_(0, this->chunk(chunk__).offset_),
                                                             this->num_gkvec_loc(), this->chunk(chunk__).num_beta_);
-                break;
-            }
-            case device_t::GPU: {
-                Beta_projectors_base<T>::generate(chunk__, 0);
-                break;
-            }
+        }
+        if (is_device_memory(mem__)) {
+            Beta_projectors_base<T>::generate(mem__, chunk__, 0);
         }
     }
 
