@@ -45,7 +45,7 @@ class Beta_projectors_gradient: public Beta_projectors_base<T>
             #pragma omp parallel for
             for (int i = 0; i < this->num_beta_t(); i++) {
                 for (int igkloc = 0; igkloc < this->num_gkvec_loc(); igkloc++) {
-                    auto vgc = this->gkvec_.template gkvec_cart<index_domain_t::local>(igkloc);
+                    auto vgc = this->gkvec_.template gkvec_cart<sddk::index_domain_t::local>(igkloc);
                     this->pw_coeffs_t_(igkloc, i, x) = std::complex<T>(0, -vgc[x]) * beta__.pw_coeffs_t(igkloc, i, 0);
                 }
             }
@@ -53,7 +53,7 @@ class Beta_projectors_gradient: public Beta_projectors_base<T>
     }
 
   public:
-    Beta_projectors_gradient(Simulation_context& ctx__, Gvec const& gkvec__, Beta_projectors<T>& beta__)
+    Beta_projectors_gradient(Simulation_context& ctx__, sddk::Gvec const& gkvec__, Beta_projectors<T>& beta__)
         : Beta_projectors_base<T>(ctx__, gkvec__, 3)
     {
         generate_pw_coefs_t(beta__);
