@@ -25,7 +25,6 @@
 #ifndef __SIMULATION_PARAMETERS_HPP__
 #define __SIMULATION_PARAMETERS_HPP__
 
-#include <mpi.h>
 #include "typedefs.hpp"
 #include "utils/cmd_args.hpp"
 #include "utils/utils.hpp"
@@ -33,8 +32,6 @@
 #include "dft/smearing.hpp"
 #include "context/config.hpp"
 
-using namespace sddk;
-
 namespace sirius {
 
 /// Get all possible options for initializing sirius. It is a json dictionary.
@@ -58,6 +55,7 @@ class Config : public config_t
 };
 
 /// Set of basic parameters of a simulation.
+/** This class provides shortcuts to the mostly used input parameters, for example `verbosity`. */
 class Simulation_parameters
 {
   private:
@@ -77,7 +75,7 @@ class Simulation_parameters
 
   protected:
     /// Type of the processing unit.
-    device_t processing_unit_{device_t::CPU};
+    sddk::device_t processing_unit_{sddk::device_t::CPU};
 
     /// Type of relativity for valence states.
     relativity_t valence_relativity_{relativity_t::zora};
@@ -91,9 +89,6 @@ class Simulation_parameters
     /// Type of occupation numbers smearing.
     smearing::smearing_t smearing_{smearing::smearing_t::gaussian};
 
-    /// Storage for various memory pools.
-    mutable std::map<memory_t, memory_pool> memory_pool_;
-
     /* copy constructor is forbidden */
     Simulation_parameters(Simulation_parameters const&) = delete;
 
@@ -162,7 +157,7 @@ class Simulation_parameters
 
     void electronic_structure_method(std::string name__);
 
-    electronic_structure_method_t electronic_structure_method() const
+    auto electronic_structure_method() const
     {
         return electronic_structure_method_;
     }
@@ -177,7 +172,7 @@ class Simulation_parameters
 
     void smearing(std::string name__);
 
-    smearing::smearing_t smearing() const
+    auto smearing() const
     {
         return smearing_;
     }
@@ -379,7 +374,7 @@ class Simulation_parameters
         return cfg().parameters().gamma_point();
     }
 
-    device_t processing_unit() const
+    sddk::device_t processing_unit() const
     {
         return processing_unit_;
     }
@@ -420,11 +415,6 @@ class Simulation_parameters
         return cfg().control().cyclic_block_size();
     }
 
-    //inline void full_potential(const bool value)
-    //{
-    //    electronic_structure_method_ = electronic_structure_method_t::full_potential_lapwlo;
-    //}
-
     bool full_potential() const
     {
         return (electronic_structure_method_ == electronic_structure_method_t::full_potential_lapwlo);
@@ -486,12 +476,14 @@ class Simulation_parameters
         return cfg().parameters().molecule();
     }
 
-    /// Get a `using symmetry` flag.
+    /// Get a use_symmetry flag.
+    /** If crystal symmetry is used, density and potential are symmetrized. */
     bool use_symmetry() const
     {
         return cfg().parameters().use_symmetry();
     }
 
+    /// Set a use_symmetry flag.
     bool use_symmetry(bool use_symmetry__)
     {
         cfg().parameters().use_symmetry(use_symmetry__);
@@ -517,32 +509,6 @@ class Simulation_parameters
         cfg_.settings().sht_coverage(sht_coverage__);
         return cfg_.settings().sht_coverage();
     }
-
-    /// Return a reference to a memory pool.
-    /** A memory pool is created when this function called for the first time. */
-    memory_pool& mem_pool(memory_t M__) const
-    {
-        if (memory_pool_.count(M__) == 0) {
-            memory_pool_.emplace(M__, memory_pool(M__));
-        }
-        return memory_pool_.at(M__);
-    }
-
-    /// Get a default memory pool for a given device.
-    memory_pool& mem_pool(device_t dev__)
-    {
-        switch (dev__) {
-            case device_t::CPU: {
-                return mem_pool(memory_t::host);
-                break;
-            }
-            case device_t::GPU: {
-                return mem_pool(memory_t::device);
-                break;
-            }
-        }
-        return mem_pool(memory_t::host); // make compiler happy
-    }
 };
 
 }; // namespace sirius
