@@ -27,20 +27,20 @@
 namespace sirius {
 
 double
-ewald_energy(const Simulation_context& ctx, const Gvec& gvec, const Unit_cell& unit_cell)
+ewald_energy(const Simulation_context& ctx, const sddk::Gvec& gvec, const Unit_cell& unit_cell)
 {
     double alpha{ctx.ewald_lambda()};
     double ewald_g{0};
 
     #pragma omp parallel for reduction(+ : ewald_g)
     for (int igloc = gvec.skip_g0(); igloc < gvec.count(); igloc++) {
-        double g2 = std::pow(gvec.gvec_len<index_domain_t::local>(igloc), 2);
+        double g2 = std::pow(gvec.gvec_len<sddk::index_domain_t::local>(igloc), 2);
 
         double_complex rho(0, 0);
 
         for (int ia = 0; ia < unit_cell.num_atoms(); ia++) {
-            rho += ctx.gvec_phase_factor(gvec.gvec<index_domain_t::local>(igloc), ia) *
-                static_cast<double>(unit_cell.atom(ia).zn());
+            rho += ctx.gvec_phase_factor(gvec.gvec<sddk::index_domain_t::local>(igloc), ia) *
+                   static_cast<double>(unit_cell.atom(ia).zn());
         }
 
         ewald_g += std::pow(std::abs(rho), 2) * std::exp(-g2 / 4 / alpha) / g2;
@@ -165,18 +165,13 @@ total_energy(Simulation_context const& ctx, K_point_set const& kset, Density con
 
         case electronic_structure_method_t::pseudopotential: {
             tot_en = (kset.valence_eval_sum() - energy_vxc(density, potential) - energy_bxc(density, potential) -
-                      potential.PAW_one_elec_energy(density)) -
+                      potential.PAW_one_elec_energy(density) - one_electron_energy_hubbard(density, potential)) -
                      0.5 * energy_vha(potential) + energy_exc(density, potential) + potential.PAW_total_energy() +
-                     ewald_energy + kset.entropy_sum();
+                     ewald_energy + kset.entropy_sum() + ::sirius::hubbard_energy(density);
             break;
         }
     }
 
-    if (ctx.hubbard_correction()) {
-        tot_en += ::sirius::energy(density.occupation_matrix());
-        tot_en -= ::sirius::one_electron_energy_hubbard(density, potential);
-    }
-
     return tot_en;
 }
 
@@ -210,11 +205,8 @@ one_electron_energy_hubbard(Density const& density, Potential const& potential)
 double
 energy_potential(Density const& density, Potential const& potential)
 {
-    double e =
-        energy_veff(density, potential) + energy_bxc(density, potential) + potential.PAW_one_elec_energy(density);
-    if (potential.ctx().hubbard_correction()) {
-        e += ::sirius::energy(density.occupation_matrix());
-    }
+    const double e = energy_veff(density, potential) + energy_bxc(density, potential) +
+                     potential.PAW_one_elec_energy(density) + ::sirius::hubbard_energy(density);
     return e;
 }
 
