@@ -34,13 +34,13 @@ namespace sirius {
 
 using namespace geometry3d;
 
-template <typename T>
+template <typename T, typename F>
 void
 Stress::calc_stress_nonloc_aux()
 {
     PROFILE("sirius::Stress|nonloc");
 
-    mdarray<real_type<T>, 2> collect_result(9, ctx_.unit_cell().num_atoms());
+    sddk::mdarray<real_type<F>, 2> collect_result(9, ctx_.unit_cell().num_atoms());
     collect_result.zero();
 
     stress_nonloc_.zero();
@@ -50,31 +50,16 @@ Stress::calc_stress_nonloc_aux()
         return;
     }
 
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
 
     for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
         int ik  = kset_.spl_num_kpoints(ikloc);
-        auto kp = kset_.get<real_type<T>>(ik);
-        if (is_device_memory(ctx_.preferred_memory_t())) {
-            int nbnd = ctx_.num_bands();
-            for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-                /* allocate GPU memory */
-                kp->spinor_wave_functions().pw_coeffs(ispn).allocate(ctx_.mem_pool(memory_t::device));
-                kp->spinor_wave_functions().pw_coeffs(ispn).copy_to(memory_t::device, 0, nbnd);
-            }
-        }
-        Beta_projectors_strain_deriv<real_type<T>> bp_strain_deriv(ctx_, kp->gkvec());
+        auto kp = kset_.get<T>(ik);
+        auto mem = ctx_.processing_unit() == sddk::device_t::CPU ? sddk::memory_t::host : sddk::memory_t::device;
+        auto mg = kp->spinor_wave_functions().memory_guard(mem, wf::copy_to::device);
+        Beta_projectors_strain_deriv<T> bp_strain_deriv(ctx_, kp->gkvec());
 
-        Non_local_functor<T> nlf(ctx_, bp_strain_deriv);
-
-        nlf.add_k_point_contribution(*kp, collect_result);
-
-        if (is_device_memory(ctx_.preferred_memory_t())) {
-            for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-                /* deallocate GPU memory */
-                kp->spinor_wave_functions().pw_coeffs(ispn).deallocate(memory_t::device);
-            }
-        }
+        add_k_point_contribution_nonlocal<T, F>(ctx_, bp_strain_deriv, *kp, collect_result);
     }
 
     #pragma omp parallel
@@ -101,10 +86,6 @@ Stress::calc_stress_nonloc_aux()
     symmetrize(stress_nonloc_);
 }
 
-template void Stress::calc_stress_nonloc_aux<double>();
-
-template void Stress::calc_stress_nonloc_aux<double_complex>();
-
 matrix3d<double>
 Stress::calc_stress_total()
 {
@@ -146,16 +127,27 @@ Stress::calc_stress_hubbard()
 
     Q_operator<double> q_op(ctx_);
 
+    auto nhwf = ctx_.unit_cell().num_hubbard_wf().first;
+
+    sddk::mdarray<double_complex, 4> dn(nhwf, nhwf, 2, 9);
+    if (is_device_memory(ctx_.processing_unit_memory_t())) {
+        dn.allocate(ctx_.processing_unit_memory_t());
+    }
     for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
         int ik  = kset_.spl_num_kpoints(ikloc);
         auto kp = kset_.get<double>(ik);
-        mdarray<double_complex, 4> dn(kp->hubbard_wave_functions_S().num_wf(), kp->hubbard_wave_functions_S().num_wf(),
-                                      2, 9);
         dn.zero();
+        if (is_device_memory(ctx_.processing_unit_memory_t())) {
+            dn.zero(ctx_.processing_unit_memory_t());
+        }
         kp->beta_projectors().prepare();
+        auto mg1 = kp->spinor_wave_functions().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
+        auto mg2 = kp->hubbard_wave_functions_S().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
+        auto mg3 = kp->atomic_wave_functions().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
+        auto mg4 = kp->atomic_wave_functions_S().memory_guard(ctx_.processing_unit_memory_t(), wf::copy_to::device);
 
         if (ctx_.num_mag_dims() == 3) {
-            TERMINATE("Hubbard stress correction is only implemented for the simple hubbard correction.");
+            RTE_THROW("Hubbard stress correction is only implemented for the simple hubbard correction.");
         }
 
         /* compute the derivative of the occupancies numbers */
@@ -251,20 +243,20 @@ Stress::calc_stress_core()
 
     auto q     = ctx_.gvec().shells_len();
     auto ff    = ctx_.ps_core_ri_djl().values(q, ctx_.comm());
-    auto drhoc = ctx_.make_periodic_function<index_domain_t::local>(ff);
+    auto drhoc = ctx_.make_periodic_function<sddk::index_domain_t::local>(ff);
 
     double sdiag{0};
     int ig0 = ctx_.gvec().skip_g0();
 
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-        auto G = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+        auto G = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
         auto g = G.length();
 
         for (int mu : {0, 1, 2}) {
             for (int nu : {0, 1, 2}) {
                 stress_core_(mu, nu) -=
-                    std::real(std::conj(potential_.xc_potential().f_pw_local(igloc)) * drhoc[igloc]) * G[mu] * G[nu] /
-                    g;
+                    std::real(std::conj(potential_.xc_potential().f_pw_local(igloc)) * drhoc[igloc]) *
+                    G[mu] * G[nu] / g;
             }
         }
 
@@ -312,7 +304,7 @@ Stress::calc_stress_xc()
            derivative of sigm (which is grad(rho) * grad(rho)) */
 
         if (ctx_.num_spins() == 1) {
-            Smooth_periodic_function<double> rhovc(ctx_.spfft<double>(), ctx_.gvec_partition());
+            Smooth_periodic_function<double> rhovc(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
             rhovc.zero();
             rhovc.add(density_.rho());
             rhovc.add(density_.rho_pseudo_core());
@@ -369,7 +361,7 @@ Stress::calc_stress_xc()
                 }
             }
         }
-        Communicator(ctx_.spfft<double>().communicator()).allreduce(&t(0, 0), 9);
+        sddk::Communicator(ctx_.spfft<double>().communicator()).allreduce(&t(0, 0), 9);
         t *= (-1.0 / ctx_.fft_grid().num_points());
         stress_xc_ += t;
     }
@@ -398,21 +390,21 @@ Stress::calc_stress_us()
 
     Augmentation_operator_gvec_deriv q_deriv(ctx_, ctx_.unit_cell().lmax(), ctx_.gvec(), ctx_.gvec_tp());
 
-    linalg_t la{linalg_t::none};
-    memory_t qmem{memory_t::none};
+    sddk::linalg_t la{sddk::linalg_t::none};
+    sddk::memory_t qmem{sddk::memory_t::none};
 
-    memory_pool* mp{nullptr};
+    sddk::memory_pool* mp{nullptr};
     switch (ctx_.processing_unit()) {
-        case device_t::CPU: {
-            mp   = &ctx_.mem_pool(memory_t::host);
-            la   = linalg_t::blas;
-            qmem = memory_t::host;
+        case sddk::device_t::CPU: {
+            mp   = &get_memory_pool(sddk::memory_t::host);
+            la   = sddk::linalg_t::blas;
+            qmem = sddk::memory_t::host;
             break;
         }
-        case device_t::GPU: {
-            mp   = &ctx_.mem_pool(memory_t::host_pinned);
-            la   = linalg_t::spla;
-            qmem = memory_t::device;
+        case sddk::device_t::GPU: {
+            mp   = &get_memory_pool(sddk::memory_t::host_pinned);
+            la   = sddk::linalg_t::spla;
+            qmem = sddk::memory_t::device;
             break;
         }
     }
@@ -431,7 +423,7 @@ Stress::calc_stress_us()
         auto dm = density_.density_matrix_aux(density_.density_matrix(), iat);
 
         sddk::mdarray<double_complex, 2> phase_factors(atom_type.num_atoms(), ctx_.gvec().count(),
-                                                       ctx_.mem_pool(memory_t::host));
+                                                       get_memory_pool(sddk::memory_t::host));
 
         PROFILE_START("sirius::Stress|us|phase_fac");
         #pragma omp parallel for schedule(static)
@@ -443,8 +435,8 @@ Stress::calc_stress_us()
         }
         PROFILE_STOP("sirius::Stress|us|phase_fac");
 
-        mdarray<double, 2> v_tmp(atom_type.num_atoms(), ctx_.gvec().count() * 2, *mp);
-        mdarray<double, 2> tmp(nbf * (nbf + 1) / 2, atom_type.num_atoms(), *mp);
+        sddk::mdarray<double, 2> v_tmp(atom_type.num_atoms(), ctx_.gvec().count() * 2, *mp);
+        sddk::mdarray<double, 2> tmp(nbf * (nbf + 1) / 2, atom_type.num_atoms(), *mp);
         /* over spin components, can be from 1 to 4 */
         for (int ispin = 0; ispin < ctx_.num_mag_dims() + 1; ispin++) {
             for (int nu = 0; nu < 3; nu++) {
@@ -461,7 +453,7 @@ Stress::calc_stress_us()
                     }
                     #pragma omp parallel for schedule(static)
                     for (int igloc = igloc0; igloc < ctx_.gvec().count(); igloc++) {
-                        auto gvc = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+                        auto gvc = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
                         double g = gvc.length();
 
                         for (int ia = 0; ia < atom_type.num_atoms(); ia++) {
@@ -474,10 +466,10 @@ Stress::calc_stress_us()
                     PROFILE_STOP("sirius::Stress|us|prepare");
 
                     PROFILE_START("sirius::Stress|us|gemm");
-                    linalg(la).gemm('N', 'T', nbf * (nbf + 1) / 2, atom_type.num_atoms(), 2 * ctx_.gvec().count(),
-                                    &linalg_const<double>::one(), q_deriv.q_pw().at(qmem), q_deriv.q_pw().ld(),
-                                    v_tmp.at(memory_t::host), v_tmp.ld(), &linalg_const<double>::zero(),
-                                    tmp.at(memory_t::host), tmp.ld());
+                    sddk::linalg(la).gemm('N', 'T', nbf * (nbf + 1) / 2, atom_type.num_atoms(), 2 * ctx_.gvec().count(),
+                                    &sddk::linalg_const<double>::one(), q_deriv.q_pw().at(qmem), q_deriv.q_pw().ld(),
+                                    v_tmp.at(sddk::memory_t::host), v_tmp.ld(), &sddk::linalg_const<double>::zero(),
+                                    tmp.at(sddk::memory_t::host), tmp.ld());
                     PROFILE_STOP("sirius::Stress|us|gemm");
 
                     for (int ia = 0; ia < atom_type.num_atoms(); ia++) {
@@ -517,7 +509,7 @@ Stress::calc_stress_ewald()
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
         int ig = ctx_.gvec().offset() + igloc;
 
-        auto G          = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+        auto G          = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
         double g2       = std::pow(G.length(), 2);
         double g2lambda = g2 / 4.0 / lambda;
 
@@ -574,66 +566,55 @@ Stress::calc_stress_ewald()
 }
 
 void
-Stress::print_info() const
+Stress::print_info(std::ostream& out__, int verbosity__) const
 {
-    if (ctx_.comm().rank() == 0) {
-
-        auto print_stress = [&](matrix3d<double> const& s) {
-            for (int mu : {0, 1, 2}) {
-                std::printf("%12.6f %12.6f %12.6f\n", s(mu, 0), s(mu, 1), s(mu, 2));
-            }
-        };
-
-        const double au2kbar = 2.94210119E5;
-        auto stress_kin      = stress_kin_ * au2kbar;
-        auto stress_har      = stress_har_ * au2kbar;
-        auto stress_ewald    = stress_ewald_ * au2kbar;
-        auto stress_vloc     = stress_vloc_ * au2kbar;
-        auto stress_xc       = stress_xc_ * au2kbar;
-        auto stress_nonloc   = stress_nonloc_ * au2kbar;
-        auto stress_us       = stress_us_ * au2kbar;
-        auto stress_hubbard  = stress_hubbard_ * au2kbar;
-        auto stress_core     = stress_core_ * au2kbar;
-
-        std::printf("== stress tensor components [kbar] ===\n");
-
-        std::printf("== stress_kin ==\n");
-        print_stress(stress_kin);
-
-        std::printf("== stress_har ==\n");
-        print_stress(stress_har);
-
-        std::printf("== stress_ewald ==\n");
-        print_stress(stress_ewald);
-
-        std::printf("== stress_vloc ==\n");
-        print_stress(stress_vloc);
-
-        std::printf("== stress_xc ==\n");
-        print_stress(stress_xc);
-
-        std::printf("== stress_core ==\n");
-        print_stress(stress_core);
-
-        std::printf("== stress_nonloc ==\n");
-        print_stress(stress_nonloc);
-
-        std::printf("== stress_us ==\n");
-        print_stress(stress_us);
-
-        stress_us = stress_us + stress_nonloc;
-        std::printf("== stress_us_nl ==\n");
-        print_stress(stress_us);
-
-        if (ctx_.hubbard_correction()) {
-            std::printf("== stress_hubbard ==\n");
-            print_stress(stress_hubbard);
+    auto print_stress = [&](std::string label__, matrix3d<double> const& s) {
+        out__ << "=== " << label__ << " ===" << std::endl;
+        for (int mu : {0, 1, 2}) {
+           out__ << utils::ffmt(12, 6) << s(mu, 0)
+                 << utils::ffmt(12, 6) << s(mu, 1)
+                 << utils::ffmt(12, 6) << s(mu, 2) << std::endl;
         }
+    };
 
-        auto stress_total = stress_total_ * au2kbar;
-        std::printf("== stress_total ==\n");
-        print_stress(stress_total);
+    const double au2kbar = 2.94210119E5;
+    auto stress_kin      = stress_kin_ * au2kbar;
+    auto stress_har      = stress_har_ * au2kbar;
+    auto stress_ewald    = stress_ewald_ * au2kbar;
+    auto stress_vloc     = stress_vloc_ * au2kbar;
+    auto stress_xc       = stress_xc_ * au2kbar;
+    auto stress_nonloc   = stress_nonloc_ * au2kbar;
+    auto stress_us       = stress_us_ * au2kbar;
+    auto stress_hubbard  = stress_hubbard_ * au2kbar;
+    auto stress_core     = stress_core_ * au2kbar;
+
+    out__ << "=== stress tensor components [kbar] ===" << std::endl;
+
+    print_stress("stress_kin", stress_kin);
+
+    print_stress("stress_har", stress_har);
+
+    print_stress("stress_ewald", stress_ewald);
+
+    print_stress("stress_vloc", stress_vloc);
+
+    print_stress("stress_xc", stress_xc);
+
+    print_stress("stress_core", stress_core);
+
+    print_stress("stress_nonloc", stress_nonloc);
+
+    print_stress("stress_us", stress_us);
+
+    stress_us = stress_us + stress_nonloc;
+    print_stress("stress_us_nl", stress_us);
+
+    if (ctx_.hubbard_correction()) {
+        print_stress("stress_hubbard", stress_hubbard);
     }
+
+    auto stress_total = stress_total_ * au2kbar;
+    print_stress("stress_total", stress_total);
 }
 
 matrix3d<double>
@@ -645,7 +626,7 @@ Stress::calc_stress_har()
 
     int ig0 = ctx_.gvec().skip_g0();
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-        auto G    = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+        auto G    = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
         double g2 = std::pow(G.length(), 2);
         auto z    = density_.rho().f_pw_local(igloc);
         double d  = twopi * (std::pow(z.real(), 2) + std::pow(z.imag(), 2)) / g2;
@@ -691,10 +672,10 @@ Stress::calc_stress_kin_aux()
                 #pragma omp for
                 for (int i = 0; i < kp->num_occupied_bands(ispin); i++) {
                     for (int igloc = 0; igloc < kp->num_gkvec_loc(); igloc++) {
-                        auto Gk = kp->gkvec().template gkvec_cart<index_domain_t::local>(igloc);
+                        auto Gk = kp->gkvec().template gkvec_cart<sddk::index_domain_t::local>(igloc);
 
                         double f = kp->band_occupancy(i, ispin);
-                        auto z = kp->spinor_wave_functions().pw_coeffs(ispin).prime(igloc, i);
+                        auto z = kp->spinor_wave_functions().pw_coeffs(igloc, wf::spin_index(ispin), wf::band_index(i));
                         double d = fact * f * (std::pow(z.real(), 2) + std::pow(z.imag(), 2));
                         for (int mu : {0, 1, 2}) {
                             for (int nu : {0, 1, 2}) {
@@ -763,15 +744,15 @@ Stress::calc_stress_vloc()
     auto ri_vloc    = ctx_.vloc_ri().values(q, ctx_.comm());
     auto ri_vloc_dg = ctx_.vloc_ri_djl().values(q, ctx_.comm());
 
-    auto v  = ctx_.make_periodic_function<index_domain_t::local>(ri_vloc);
-    auto dv = ctx_.make_periodic_function<index_domain_t::local>(ri_vloc_dg);
+    auto v  = ctx_.make_periodic_function<sddk::index_domain_t::local>(ri_vloc);
+    auto dv = ctx_.make_periodic_function<sddk::index_domain_t::local>(ri_vloc_dg);
 
     double sdiag{0};
 
     int ig0 = ctx_.gvec().skip_g0();
     for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
 
-        auto G = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc);
+        auto G = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc);
 
         for (int mu : {0, 1, 2}) {
             for (int nu : {0, 1, 2}) {
@@ -808,18 +789,18 @@ Stress::calc_stress_nonloc()
     if (ctx_.cfg().parameters().precision_wf() == "fp32") {
 #if defined(USE_FP32)
         if (ctx_.gamma_point()) {
-            calc_stress_nonloc_aux<float>();
+            calc_stress_nonloc_aux<float, float>();
         } else {
-            calc_stress_nonloc_aux<std::complex<float>>();
+            calc_stress_nonloc_aux<float, std::complex<float>>();
         }
 #else
         RTE_THROW("Not compiled with FP32 support");
 #endif
     } else {
         if (ctx_.gamma_point()) {
-            calc_stress_nonloc_aux<double>();
+            calc_stress_nonloc_aux<double, double>();
         } else {
-            calc_stress_nonloc_aux<std::complex<double>>();
+            calc_stress_nonloc_aux<double, std::complex<double>>();
         }
     }
 
