@@ -215,7 +215,7 @@ class Stress
      *
      * \tparam T  One of float, double, complex<float> or complex<double> types for generic or Gamma point case.
      */
-    template <typename T>
+    template <typename T, typename F>
     void calc_stress_nonloc_aux();
 
     void symmetrize(matrix3d<double>& mtrx__) const;
@@ -482,7 +482,12 @@ class Stress
 
     matrix3d<double> calc_stress_total();
 
-    void print_info() const;
+    inline matrix3d<double> stress_total() const
+    {
+        return stress_total_;
+    }
+
+    void print_info(std::ostream& out__, int verbosity__) const;
 };
 
 } // namespace sirius
