@@ -33,12 +33,6 @@
 #include "context/simulation_context.hpp"
 #include "hubbard/hubbard_matrix.hpp"
 
-namespace sddk {
-template <typename T>
-class Wave_functions;
-class spin_range;
-}; // namespace sddk
-
 namespace sirius {
 /* forward declaration */
 template <typename T>
@@ -77,9 +71,12 @@ class Non_local_operator
     Non_local_operator(Simulation_context const& ctx__);
 
     /// Apply chunk of beta-projectors to all wave functions.
-    template <typename F, std::enable_if_t<std::is_same<T, F>::value, bool> = true>
-    void apply(int chunk__, int ispn_block__, Wave_functions<T>& op_phi__, int idx0__, int n__,
-               Beta_projectors_base<T>& beta__, matrix<F>& beta_phi__)
+    /** \tparam F  Type of the subspace matrix
+     */
+    template <typename F>
+    void
+    apply(sddk::memory_t mem__, int chunk__, int ispn_block__, wf::Wave_functions<T>& op_phi__, wf::band_range br__,
+          Beta_projectors_base<T> const& beta__, sddk::matrix<F> const& beta_phi__) const
     {
         PROFILE("sirius::Non_local_operator::apply");
 
@@ -87,108 +84,24 @@ class Non_local_operator
             return;
         }
 
-        auto& beta_gk     = beta__.pw_coeffs_a();
-        int num_gkvec_loc = beta__.num_gkvec_loc();
-        int nbeta         = beta__.chunk(chunk__).num_beta_;
+        auto const& beta_gk = beta__.pw_coeffs_a();
+        int num_gkvec_loc   = beta__.num_gkvec_loc();
+        int nbeta           = beta__.chunk(chunk__).num_beta_;
 
         /* setup linear algebra parameters */
-        memory_t mem{memory_t::none};
-        linalg_t la{linalg_t::none};
-        switch (pu_) {
-            case device_t::CPU: {
-                mem = memory_t::host;
-                la  = linalg_t::blas;
-                break;
-            }
-            case device_t::GPU: {
-                mem = memory_t::device;
-                la  = linalg_t::gpublas;
-                break;
-            }
+        sddk::linalg_t la{sddk::linalg_t::blas};
+        sddk::device_t pu{sddk::device_t::CPU};
+        if (is_device_memory(mem__)) {
+            la = sddk::linalg_t::gpublas;
+            pu = sddk::device_t::GPU;
         }
 
-        auto work = mdarray<T, 1>(nbeta * n__, ctx_.mem_pool(mem));
-
-        /* compute O * <beta|phi> for atoms in a chunk */
-        #pragma omp parallel for
-        for (int i = 0; i < beta__.chunk(chunk__).num_atoms_; i++) {
-            /* number of beta functions for a given atom */
-            int nbf  = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::nbf), i);
-            int offs = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::offset), i);
-            int ia   = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::ia), i);
-
-            if (nbf == 0) {
-                continue;
-            }
-            linalg(la).gemm('N', 'N', nbf, n__, nbf, &linalg_const<T>::one(),
-                            op_.at(mem, 0, packed_mtrx_offset_(ia), ispn_block__), nbf, beta_phi__.at(mem, offs, 0),
-                            beta_phi__.ld(), &linalg_const<T>::zero(), work.at(mem, offs), nbeta,
-                            stream_id(omp_get_thread_num()));
-        }
-        switch (pu_) {
-            case device_t::GPU: {
-                /* wait for previous zgemms */
-                #pragma omp parallel
-                acc::sync_stream(stream_id(omp_get_thread_num()));
-                break;
-            }
-            case device_t::CPU: {
-                break;
-            }
+        int size_factor = 1;
+        if (std::is_same<F, real_type<F>>::value) {
+            size_factor = 2;
         }
 
-        int jspn = ispn_block__ & 1;
-
-        /* compute <G+k|beta> * O * <beta|phi> and add to op_phi */
-        linalg(ctx_.blas_linalg_t())
-            .gemm('N', 'N', 2 * num_gkvec_loc, n__, nbeta, &linalg_const<T>::one(),
-                  reinterpret_cast<T*>(beta_gk.at(mem)), 2 * num_gkvec_loc, work.at(mem), nbeta,
-                  &linalg_const<T>::one(),
-                  reinterpret_cast<T*>(op_phi__.pw_coeffs(jspn).prime().at(op_phi__.preferred_memory_t(), 0, idx0__)),
-                  2 * op_phi__.pw_coeffs(jspn).prime().ld());
-
-        switch (pu_) {
-            case device_t::GPU: {
-                acc::sync_stream(stream_id(-1));
-                break;
-            }
-            case device_t::CPU: {
-                break;
-            }
-        }
-    }
-
-    template <typename F, std::enable_if_t<std::is_same<std::complex<T>, F>::value, bool> = true>
-    void apply(int chunk__, int ispn_block__, Wave_functions<T>& op_phi__, int idx0__, int n__,
-               Beta_projectors_base<T>& beta__, matrix<F>& beta_phi__)
-    {
-        PROFILE("sirius::Non_local_operator::apply");
-
-        if (is_null_) {
-            return;
-        }
-
-        auto& beta_gk     = beta__.pw_coeffs_a();
-        int num_gkvec_loc = beta__.num_gkvec_loc();
-        int nbeta         = beta__.chunk(chunk__).num_beta_;
-
-        /* setup linear algebra parameters */
-        memory_t mem{memory_t::none};
-        linalg_t la{linalg_t::none};
-        switch (pu_) {
-            case device_t::CPU: {
-                mem = memory_t::host;
-                la  = linalg_t::blas;
-                break;
-            }
-            case device_t::GPU: {
-                mem = memory_t::device;
-                la  = linalg_t::gpublas;
-                break;
-            }
-        }
-
-        auto work = mdarray<std::complex<T>, 1>(nbeta * n__, ctx_.mem_pool(mem));
+        auto work = sddk::mdarray<F, 2>(nbeta, br__.size(), get_memory_pool(mem__));
 
         /* compute O * <beta|phi> for atoms in a chunk */
         #pragma omp parallel
@@ -198,55 +111,58 @@ class Non_local_operator
             #pragma omp for
             for (int i = 0; i < beta__.chunk(chunk__).num_atoms_; i++) {
                 /* number of beta functions for a given atom */
-                int nbf  = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::nbf), i);
-                int offs = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::offset), i);
-                int ia   = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::ia), i);
+                int nbf  = beta__.chunk(chunk__).desc_(beta_desc_idx::nbf, i);
+                int offs = beta__.chunk(chunk__).desc_(beta_desc_idx::offset, i);
+                int ia   = beta__.chunk(chunk__).desc_(beta_desc_idx::ia, i);
 
                 if (nbf) {
-                    linalg(la).gemm(
-                        'N', 'N', nbf, n__, nbf, &linalg_const<std::complex<T>>::one(),
-                        reinterpret_cast<std::complex<T>*>(op_.at(mem, 0, packed_mtrx_offset_(ia), ispn_block__)), nbf,
-                        beta_phi__.at(mem, offs, 0), beta_phi__.ld(), &linalg_const<std::complex<T>>::zero(),
-                        work.at(mem, offs), nbeta, stream_id(omp_get_thread_num()));
+                    sddk::linalg(la).gemm(
+                        'N', 'N', nbf, br__.size(), nbf, &sddk::linalg_const<F>::one(),
+                        reinterpret_cast<F const*>(op_.at(mem__, 0, packed_mtrx_offset_(ia), ispn_block__)), nbf,
+                        reinterpret_cast<F const*>(beta_phi__.at(mem__, offs, 0)), beta_phi__.ld(),
+                        &sddk::linalg_const<F>::zero(),
+                        reinterpret_cast<F*>(work.at(mem__, offs, 0)), nbeta, stream_id(omp_get_thread_num()));
                 }
             }
         }
-        switch (pu_) {
-            case device_t::GPU: {
+        switch (pu) { // TODO: check if this is needed. Null stream later should sync the streams.
+            case sddk::device_t::GPU: {
                 /* wait for previous zgemms */
                 #pragma omp parallel
                 acc::sync_stream(stream_id(omp_get_thread_num()));
                 break;
             }
-            case device_t::CPU: {
+            case sddk::device_t::CPU: {
                 break;
             }
         }
 
-        int jspn = ispn_block__ & 1;
+        auto sp = op_phi__.actual_spin_index(wf::spin_index(ispn_block__ & 1));
 
         /* compute <G+k|beta> * O * <beta|phi> and add to op_phi */
-        linalg(ctx_.blas_linalg_t())
-            .gemm('N', 'N', num_gkvec_loc, n__, nbeta, &linalg_const<std::complex<T>>::one(), beta_gk.at(mem),
-                  num_gkvec_loc, work.at(mem), nbeta, &linalg_const<std::complex<T>>::one(),
-                  op_phi__.pw_coeffs(jspn).prime().at(op_phi__.preferred_memory_t(), 0, idx0__),
-                  op_phi__.pw_coeffs(jspn).prime().ld());
+        sddk::linalg(la)
+            .gemm('N', 'N', num_gkvec_loc * size_factor, br__.size(), nbeta, &sddk::linalg_const<F>::one(),
+                reinterpret_cast<F const*>(beta_gk.at(mem__)), num_gkvec_loc * size_factor, 
+                work.at(mem__), nbeta, &sddk::linalg_const<F>::one(),
+                reinterpret_cast<F*>(op_phi__.at(mem__, 0, sp, wf::band_index(br__.begin()))),
+                op_phi__.ld() * size_factor);
 
-        switch (pu_) {
-            case device_t::GPU: {
+        switch (pu) {
+            case sddk::device_t::GPU: {
                 acc::sync_stream(stream_id(-1));
                 break;
             }
-            case device_t::CPU: {
+            case sddk::device_t::CPU: {
                 break;
             }
         }
     }
 
     /// Apply beta projectors from one atom in a chunk of beta projectors to all wave-functions.
-    template <typename F, std::enable_if_t<std::is_same<std::complex<T>, F>::value, bool> = true>
-    void apply(int chunk__, int ia__, int ispn_block__, Wave_functions<T>& op_phi__, int idx0__, int n__,
-               Beta_projectors_base<T>& beta__, matrix<F>& beta_phi__)
+    template <typename F>
+    std::enable_if_t<std::is_same<std::complex<T>, F>::value, void>
+    apply(sddk::memory_t mem__, int chunk__, wf::atom_index ia__, int ispn_block__, wf::Wave_functions<T>& op_phi__,
+            wf::band_range br__, Beta_projectors_base<T>& beta__, sddk::matrix<F>& beta_phi__)
     {
         if (is_null_) {
             return;
@@ -255,50 +171,41 @@ class Non_local_operator
         auto& beta_gk     = beta__.pw_coeffs_a();
         int num_gkvec_loc = beta__.num_gkvec_loc();
 
-        int nbf  = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::nbf), ia__);
-        int offs = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::offset), ia__);
-        int ia   = beta__.chunk(chunk__).desc_(static_cast<int>(beta_desc_idx::ia), ia__);
+        int nbf  = beta__.chunk(chunk__).desc_(beta_desc_idx::nbf, ia__.get());
+        int offs = beta__.chunk(chunk__).desc_(beta_desc_idx::offset, ia__.get());
+        int ia   = beta__.chunk(chunk__).desc_(beta_desc_idx::ia, ia__.get());
 
         if (nbf == 0) {
             return;
         }
 
-        /* setup linear algebra parameters */
-        memory_t mem{memory_t::none};
-        linalg_t la{linalg_t::none};
-
-        switch (pu_) {
-            case device_t::CPU: {
-                mem = memory_t::host;
-                la  = linalg_t::blas;
-                break;
-            }
-            case device_t::GPU: {
-                mem = memory_t::device;
-                la  = linalg_t::gpublas;
-                break;
-            }
+        sddk::linalg_t la{sddk::linalg_t::blas};
+        sddk::device_t pu{sddk::device_t::CPU};
+        if (is_device_memory(mem__)) {
+            la = sddk::linalg_t::gpublas;
+            pu = sddk::device_t::GPU;
         }
 
-        auto work = mdarray<std::complex<T>, 1>(nbf * n__, ctx_.mem_pool(mem));
+        auto work = sddk::mdarray<std::complex<T>, 1>(nbf * br__.size(), get_memory_pool(mem__));
 
-        linalg(la).gemm('N', 'N', nbf, n__, nbf, &linalg_const<std::complex<T>>::one(),
-                        reinterpret_cast<std::complex<T>*>(op_.at(mem, 0, packed_mtrx_offset_(ia), ispn_block__)), nbf,
-                        beta_phi__.at(mem, offs, 0), beta_phi__.ld(), &linalg_const<std::complex<T>>::zero(),
-                        work.at(mem), nbf);
+        sddk::linalg(la).gemm('N', 'N', nbf, br__.size(), nbf, &sddk::linalg_const<std::complex<T>>::one(),
+                        reinterpret_cast<std::complex<T>*>(op_.at(mem__, 0, packed_mtrx_offset_(ia), ispn_block__)), nbf,
+                        beta_phi__.at(mem__, offs, 0), beta_phi__.ld(), &sddk::linalg_const<std::complex<T>>::zero(),
+                        work.at(mem__), nbf);
 
         int jspn = ispn_block__ & 1;
 
-        linalg(ctx_.blas_linalg_t())
-            .gemm('N', 'N', num_gkvec_loc, n__, nbf, &linalg_const<std::complex<T>>::one(), beta_gk.at(mem, 0, offs),
-                  num_gkvec_loc, work.at(mem), nbf, &linalg_const<std::complex<T>>::one(),
-                  op_phi__.pw_coeffs(jspn).prime().at(op_phi__.preferred_memory_t(), 0, idx0__),
-                  op_phi__.pw_coeffs(jspn).prime().ld());
-        switch (pu_) {
-            case device_t::CPU: {
+        sddk::linalg(la)
+            .gemm('N', 'N', num_gkvec_loc, br__.size(), nbf, &sddk::linalg_const<std::complex<T>>::one(),
+                    beta_gk.at(mem__, 0, offs), num_gkvec_loc, work.at(mem__), nbf, &sddk::linalg_const<std::complex<T>>::one(),
+                  op_phi__.at(mem__, 0, wf::spin_index(jspn), wf::band_index(br__.begin())),
+                  op_phi__.ld());
+
+        switch (pu) {
+            case sddk::device_t::CPU: {
                 break;
             }
-            case device_t::GPU: {
+            case sddk::device_t::GPU: {
 #ifdef SIRIUS_GPU
                 acc::sync_stream(stream_id(-1));
 #endif
@@ -432,8 +339,8 @@ class U_operator
             if (ctx_.print_checksum()) {
                 utils::print_checksum("um" + std::to_string(is), um_[is].checksum(r.first, r.first), RTE_OUT(ctx_.out()));
             }
-            if (ctx_.processing_unit() == device_t::GPU) {
-                um_[is].allocate(ctx_.mem_pool(memory_t::device)).copy_to(memory_t::device);
+            if (ctx_.processing_unit() == sddk::device_t::GPU) {
+                um_[is].allocate(get_memory_pool(sddk::memory_t::device)).copy_to(sddk::memory_t::device);
             }
         }
     }
@@ -461,12 +368,12 @@ class U_operator
         return offset_[ia__];
     }
 
-    std::complex<T> operator()(int m1, int m2, int j)
+    auto operator()(int m1, int m2, int j)
     {
         return um_[j](m1, m2);
     }
 
-    std::complex<T>* at(memory_t mem__, const int idx1, const int idx2, const int idx3)
+    auto* at(sddk::memory_t mem__, const int idx1, const int idx2, const int idx3)
     {
         return um_[idx3].at(mem__, idx1, idx2);
     }
@@ -536,38 +443,159 @@ class U_operator
  *  \param [in]  q_op    Pointer to Q-operator.
  *  \param [out] sphi    Resulting |beta>Q<beta|phi>
  */
-template <typename T>
-void apply_non_local_d_q(sddk::spin_range spins__, int N__, int n__, Beta_projectors<real_type<T>>& beta__,
-                         sddk::Wave_functions<real_type<T>>& phi__, D_operator<real_type<T>>* d_op__,
-                         sddk::Wave_functions<real_type<T>>* hphi__, Q_operator<real_type<T>>* q_op__,
-                         sddk::Wave_functions<real_type<T>>* sphi__);
+//template <typename T>
+//void apply_non_local_d_q(sddk::spin_range spins__, int N__, int n__, Beta_projectors<real_type<T>>& beta__,
+//                         sddk::Wave_functions<real_type<T>>& phi__, D_operator<real_type<T>>* d_op__,
+//                         sddk::Wave_functions<real_type<T>>* hphi__, Q_operator<real_type<T>>* q_op__,
+//                         sddk::Wave_functions<real_type<T>>* sphi__);
 
-template <typename T>
-void apply_S_operator(sddk::device_t pu__, sddk::spin_range spins__, int N__, int n__,
-                      Beta_projectors<real_type<T>>& beta__, sddk::Wave_functions<real_type<T>>& phi__,
-                      Q_operator<real_type<T>>* q_op__, sddk::Wave_functions<real_type<T>>& sphi__);
+/** \tparam T  Precision of the wave-functions.
+ *  \tparam F  Type of the subspace.
+ **/
+template <typename T, typename F>
+void
+apply_non_local_D_Q(sddk::memory_t mem__, wf::spin_range spins__, wf::band_range br__, Beta_projectors<T>& beta__,
+    wf::Wave_functions<T> const& phi__, D_operator<T> const* d_op__, wf::Wave_functions<T>* hphi__,
+    Q_operator<T> const* q_op__, wf::Wave_functions<T>* sphi__)
+{
+    for (int i = 0; i < beta__.num_chunks(); i++) {
+        /* generate beta-projectors for a block of atoms */
+        beta__.generate(mem__, i);
 
+        for (auto s = spins__.begin(); s != spins__.end(); s++) {
+            auto sp = phi__.actual_spin_index(s);
+            auto beta_phi = beta__.template inner<F>(mem__, i, phi__, sp, br__);
+
+            if (hphi__ && d_op__) {
+                /* apply diagonal spin blocks */
+                d_op__->apply(mem__, i, s.get(), *hphi__, br__, beta__, beta_phi);
+                if (!d_op__->is_diag() && hphi__->num_md() == wf::num_mag_dims(3)) {
+                    /* apply non-diagonal spin blocks */
+                    /* xor 3 operator will map 0 to 3 and 1 to 2 */
+                    d_op__->apply(mem__, i, s.get() ^ 3, *hphi__, br__, beta__, beta_phi);
+                }
+            }
+
+            if (sphi__ && q_op__) {
+                /* apply Q operator (diagonal in spin) */
+                q_op__->apply(mem__, i, s.get(), *sphi__, br__, beta__, beta_phi);
+                if (!q_op__->is_diag() && sphi__->num_md() == wf::num_mag_dims(3)) {
+                    q_op__->apply(mem__, i, s.get() ^ 3, *sphi__, br__, beta__, beta_phi);
+                }
+            }
+        }
+    }
+}
+
+/// Compute |sphi> = (1 + Q)|phi>
+template <typename T, typename F>
+void apply_S_operator(sddk::memory_t mem__, wf::spin_range spins__, wf::band_range br__,
+                      Beta_projectors<T>& beta__, wf::Wave_functions<T> const& phi__,
+                      Q_operator<T> const* q_op__, wf::Wave_functions<T>& sphi__)
+{
+    for (auto s = spins__.begin(); s != spins__.end(); s++) {
+        wf::copy(mem__, phi__, s, br__, sphi__, s, br__);
+    }
+
+    if (q_op__) {
+        apply_non_local_D_Q<T, F>(mem__, spins__, br__, beta__, phi__, nullptr, nullptr, q_op__, &sphi__);
+    }
+}
+
+/** Apply Hubbard U correction 
+ * \tparam T  Precision type of wave-functions (flat or double).
+ * \param [in]  hub_wf   Hubbard atomic wave-functions.
+ * \param [in]  phi      Set of wave-functions to which Hubbard correction is applied.
+ * \param [out] hphi     Output wave-functions to which the result is added.
+ */
 template <typename T>
-void apply_U_operator(Simulation_context& ctx__, spin_range spins__, int N__, int n__, Wave_functions<T>& hub_wf__,
-                      Wave_functions<T>& phi__, U_operator<T>& um__, Wave_functions<T>& hphi__);
+void
+apply_U_operator(Simulation_context& ctx__, wf::spin_range spins__, wf::band_range br__,
+        wf::Wave_functions<T> const& hub_wf__, wf::Wave_functions<T> const& phi__, U_operator<T>& um__,
+        wf::Wave_functions<T>& hphi__)
+{
+    if (!ctx__.hubbard_correction()) {
+        return;
+    }
+
+    sddk::dmatrix<std::complex<T>> dm(hub_wf__.num_wf().get(), br__.size());
+
+    auto mt = ctx__.processing_unit_memory_t();
+    auto la = sddk::linalg_t::blas;
+    if (is_device_memory(mt)) {
+        la = sddk::linalg_t::gpublas;
+        dm.allocate(mt);
+    }
+
+    /* First calculate the local part of the projections
+       dm(i, n) = <phi_i| S |psi_{nk}> */
+    wf::inner(ctx__.spla_context(), mt, spins__, hub_wf__, wf::band_range(0, hub_wf__.num_wf().get()), phi__, br__, dm, 0, 0);
+
+    sddk::dmatrix<std::complex<T>> Up(hub_wf__.num_wf().get(), br__.size());
+    if (is_device_memory(mt)) {
+        Up.allocate(mt);
+    }
+
+    if (ctx__.num_mag_dims() == 3) {
+        Up.zero();
+        #pragma omp parallel for schedule(static)
+        for (int at_lvl = 0; at_lvl < (int)um__.atomic_orbitals().size(); at_lvl++) {
+            const int ia     = um__.atomic_orbitals(at_lvl).first;
+            auto const& atom = ctx__.unit_cell().atom(ia);
+            if (atom.type().lo_descriptor_hub(um__.atomic_orbitals(at_lvl).second).use_for_calculation()) {
+                const int lmax_at = 2 * atom.type().lo_descriptor_hub(um__.atomic_orbitals(at_lvl).second).l() + 1;
+                // we apply the hubbard correction. For now I have no papers
+                // giving me the formula for the SO case so I rely on QE for it
+                // but I do not like it at all
+                for (int s1 = 0; s1 < ctx__.num_spins(); s1++) {
+                    for (int s2 = 0; s2 < ctx__.num_spins(); s2++) {
+                      // TODO: replace this with matrix matrix multiplication
+                        for (int nbd = 0; nbd < br__.size(); nbd++) {
+                            for (int m1 = 0; m1 < lmax_at; m1++) {
+                                for (int m2 = 0; m2 < lmax_at; m2++) {
+                                    const int ind = (s1 == s2) * s1 + (1 + 2 * s2 + s1) * (s1 != s2);
+                                    Up(um__.nhwf() * s1 + um__.offset(at_lvl) + m1, nbd) +=
+                                      um__(um__.offset(at_lvl) + m2, um__.offset(at_lvl) + m1, ind) *
+                                      dm(um__.nhwf() * s2 + um__.offset(at_lvl) + m2, nbd);
+                                }
+                            }
+                        }
+                    }
+                }
+            }
+        }
+    } else {
+        sddk::linalg(la).gemm('N', 'N', um__.nhwf(), br__.size(), um__.nhwf(),
+                &sddk::linalg_const<std::complex<T>>::one(), um__.at(mt, 0, 0, spins__.begin().get()), um__.nhwf(),
+                dm.at(mt, 0, 0), dm.ld(), &sddk::linalg_const<std::complex<T>>::zero(), Up.at(mt, 0, 0), Up.ld());
+        if (is_device_memory(mt)) {
+            Up.copy_to(sddk::memory_t::host);
+        }
+    }
+    for (auto s = spins__.begin(); s != spins__.end(); s++) {
+        auto sp = hub_wf__.actual_spin_index(s);
+        wf::transform(ctx__.spla_context(), mt, Up, 0, 0, 1.0, hub_wf__, sp, wf::band_range(0, hub_wf__.num_wf().get()),
+            1.0, hphi__, sp, br__);
+    }
+}
 
 /// Apply strain derivative of S-operator to all scalar functions.
 inline void
-apply_S_operator_strain_deriv(sddk::device_t pu__, int comp__, Beta_projectors<double>& bp__,
-                              Beta_projectors_strain_deriv<double>& bp_strain_deriv__, Wave_functions<double>& phi__,
-                              Q_operator<double>& q_op__, Wave_functions<double>& ds_phi__)
+apply_S_operator_strain_deriv(sddk::memory_t mem__, int comp__, Beta_projectors<double>& bp__,
+                              Beta_projectors_strain_deriv<double>& bp_strain_deriv__, wf::Wave_functions<double>& phi__,
+                              Q_operator<double>& q_op__, wf::Wave_functions<double>& ds_phi__)
 {
     RTE_ASSERT(ds_phi__.num_wf() == phi__.num_wf());
-    //ds_phi__.zero(pu__);
     for (int ichunk = 0; ichunk < bp__.num_chunks(); ichunk++) {
         /* generate beta-projectors for a block of atoms */
-        bp__.generate(ichunk);
+        bp__.generate(mem__, ichunk);
         /* generate derived beta-projectors for a block of atoms */
-        bp_strain_deriv__.generate(ichunk, comp__);
-        auto dbeta_phi = bp_strain_deriv__.inner<double_complex>(ichunk, phi__, 0, 0, phi__.num_wf());
-        auto beta_phi = bp__.inner<double_complex>(ichunk, phi__, 0, 0, phi__.num_wf());
-        q_op__.apply(ichunk, 0, ds_phi__, 0, ds_phi__.num_wf(), bp__, dbeta_phi);
-        q_op__.apply(ichunk, 0, ds_phi__, 0, ds_phi__.num_wf(), bp_strain_deriv__, beta_phi);
+        bp_strain_deriv__.generate(mem__, ichunk, comp__);
+        auto dbeta_phi = bp_strain_deriv__.inner<double_complex>(mem__, ichunk, phi__, wf::spin_index(0),
+                wf::band_range(0, phi__.num_wf().get()));
+        auto beta_phi = bp__.inner<double_complex>(mem__, ichunk, phi__, wf::spin_index(0), wf::band_range(0, phi__.num_wf().get()));
+        q_op__.apply(mem__, ichunk, 0, ds_phi__, wf::band_range(0, ds_phi__.num_wf().get()), bp__, dbeta_phi);
+        q_op__.apply(mem__, ichunk, 0, ds_phi__, wf::band_range(0, ds_phi__.num_wf().get()), bp_strain_deriv__, beta_phi);
     }
 }
 
