@@ -27,7 +27,7 @@
 
 #include "lapw/matching_coefficients.hpp"
 #include "beta_projectors/beta_projectors.hpp"
-#include "wave_functions.hpp"
+#include "unit_cell/radial_functions_index.hpp"
 #include "SDDK/fft.hpp"
 
 namespace sirius {
@@ -59,14 +59,14 @@ class K_point
     sddk::Communicator const& comm_;
 
     /// List of G-vectors with |G+k| < cutoff.
-    std::shared_ptr<Gvec> gkvec_;
+    std::shared_ptr<sddk::Gvec> gkvec_;
 
-    std::shared_ptr<Gvec> gkvec_row_;
+    std::shared_ptr<sddk::Gvec> gkvec_row_;
 
-    std::shared_ptr<Gvec> gkvec_col_;
+    std::shared_ptr<sddk::Gvec> gkvec_col_;
 
     /// G-vector distribution for the FFT transformation.
-    std::unique_ptr<Gvec_partition> gkvec_partition_;
+    std::shared_ptr<sddk::Gvec_fft> gkvec_partition_;
 
     std::unique_ptr<spfft_transform_type<T>> spfft_transform_;
 
@@ -74,40 +74,40 @@ class K_point
     sddk::mdarray<double, 1> fv_eigen_values_;
 
     /// First-variational eigen vectors, distributed over 2D BLACS grid.
-    dmatrix<std::complex<T>> fv_eigen_vectors_;
+    sddk::dmatrix<std::complex<T>> fv_eigen_vectors_;
 
     /// First-variational eigen vectors, distributed in slabs.
-    std::unique_ptr<Wave_functions<T>> fv_eigen_vectors_slab_;
+    std::unique_ptr<wf::Wave_functions<T>> fv_eigen_vectors_slab_;
 
     /// Lowest eigen-vectors of the LAPW overlap matrix with small aigen-values.
-    std::unique_ptr<Wave_functions<T>> singular_components_;
+    std::unique_ptr<wf::Wave_functions<T>> singular_components_;
 
     /// Second-variational eigen vectors.
     /** Second-variational eigen-vectors are stored as one or two \f$ N_{fv} \times N_{fv} \f$ matrices in
      *  case of non-magnetic or collinear magnetic case or as a single \f$ 2 N_{fv} \times 2 N_{fv} \f$
      *  matrix in case of general non-collinear magnetism. */
-    dmatrix<std::complex<T>> sv_eigen_vectors_[2];
+    std::array<sddk::dmatrix<std::complex<T>>, 2> sv_eigen_vectors_;
 
     /// Full-diagonalization eigen vectors.
-    mdarray<std::complex<T>, 2> fd_eigen_vectors_;
+    sddk::mdarray<std::complex<T>, 2> fd_eigen_vectors_;
 
     /// First-variational states.
-    std::unique_ptr<Wave_functions<T>> fv_states_{nullptr};
+    std::unique_ptr<wf::Wave_functions<T>> fv_states_{nullptr};
 
     /// Two-component (spinor) wave functions describing the bands.
-    std::shared_ptr<Wave_functions<T>> spinor_wave_functions_{nullptr};
+    std::unique_ptr<wf::Wave_functions<T>> spinor_wave_functions_{nullptr};
 
     /// Pseudopotential atmoic wave-functions (not orthogonalized).
-    std::unique_ptr<Wave_functions<T>> atomic_wave_functions_{nullptr};
+    std::unique_ptr<wf::Wave_functions<T>> atomic_wave_functions_{nullptr};
 
     /// Pseudopotential atmoic wave-functions (not orthogonalized) with S-operator applied.
-    std::unique_ptr<Wave_functions<T>> atomic_wave_functions_S_{nullptr};
+    std::unique_ptr<wf::Wave_functions<T>> atomic_wave_functions_S_{nullptr};
 
     /// Hubbard wave functions.
-    std::unique_ptr<Wave_functions<T>> hubbard_wave_functions_{nullptr};
+    std::unique_ptr<wf::Wave_functions<T>> hubbard_wave_functions_{nullptr};
 
     /// Hubbard wave functions with S-operator applied.
-    std::unique_ptr<Wave_functions<T>> hubbard_wave_functions_S_{nullptr};
+    std::unique_ptr<wf::Wave_functions<T>> hubbard_wave_functions_S_{nullptr};
 
     /// Band occupation numbers.
     sddk::mdarray<double, 2> band_occupancies_;
@@ -188,22 +188,19 @@ class K_point
     std::unique_ptr<Beta_projectors<T>> beta_projectors_col_{nullptr};
 
     /// Preconditioner matrix for Chebyshev solver.
-    mdarray<std::complex<T>, 3> p_mtrx_;
+    sddk::mdarray<std::complex<T>, 3> p_mtrx_;
 
     /// Communicator between(!!) rows.
-    Communicator const& comm_row_;
+    sddk::Communicator const& comm_row_;
 
     /// Communicator between(!!) columns.
-    Communicator const& comm_col_;
+    sddk::Communicator const& comm_col_;
 
-    std::array<int, 2> ispn_map_;
+    std::array<int, 2> ispn_map_{0, -1};
 
     /// Generate G+k and local orbital basis sets.
     void generate_gklo_basis();
 
-    /// Test orthonormalization of first-variational states.
-    void test_fv_states();
-
     /// Find G+k vectors within the cutoff.
     void generate_gkvec(double gk_cutoff__);
 
@@ -218,14 +215,12 @@ class K_point
     void init0()
     {
         band_occupancies_ = sddk::mdarray<double, 2>(ctx_.num_bands(), ctx_.num_spinors(),
-                                                     memory_t::host, "band_occupancies");
+                                                     sddk::memory_t::host, "band_occupancies");
         band_occupancies_.zero();
         band_energies_ = sddk::mdarray<double, 2>(ctx_.num_bands(), ctx_.num_spinors(),
-                                                  memory_t::host, "band_energies");
+                                                  sddk::memory_t::host, "band_energies");
         band_energies_.zero();
 
-        ispn_map_[0] = 0;
-        ispn_map_[1] = -1;
         if (ctx_.num_mag_dims() == 1) {
             ispn_map_[1] = 1;
         } else if (ctx_.num_mag_dims() == 3) {
@@ -249,12 +244,12 @@ class K_point
         , comm_col_(ctx_.blacs_grid().comm_col())
     {
         this->init0();
-        gkvec_ = std::make_shared<Gvec>(vk_, unit_cell_.reciprocal_lattice_vectors(), ctx_.gk_cutoff(), comm_,
+        gkvec_ = std::make_shared<sddk::Gvec>(vk_, unit_cell_.reciprocal_lattice_vectors(), ctx_.gk_cutoff(), comm_,
                                         ctx_.gamma_point());
     }
 
     /// Constructor
-    K_point(Simulation_context& ctx__, std::shared_ptr<Gvec> gkvec__, double weight__)
+    K_point(Simulation_context& ctx__, std::shared_ptr<sddk::Gvec> gkvec__, double weight__)
         : ctx_(ctx__)
         , unit_cell_(ctx_.unit_cell())
         , vk_(gkvec__->vk())
@@ -344,29 +339,24 @@ class K_point
                             sufficient storage space.
      */
     void generate_atomic_wave_functions(std::vector<int> atoms__,
-                                        std::function<sirius::experimental::basis_functions_index const*(int)> indexb__,
-                                        Radial_integrals_atomic_wf<false> const& ri__,
-                                        sddk::Wave_functions<T>& wf__);
-
+                                        std::function<experimental::basis_functions_index const*(int)> indexb__,
+                                        Radial_integrals_atomic_wf<false> const& ri__, wf::Wave_functions<T>& wf__);
     void generate_hubbard_orbitals();
 
     /// Save data to HDF5 file.
     void save(std::string const& name__, int id__) const;
 
-    void load(HDF5_tree h5in, int id);
+    void load(sddk::HDF5_tree h5in, int id);
 
     //== void save_wave_functions(int id);
 
     //== void load_wave_functions(int id);
 
     /// Collect distributed first-variational vectors into a global array.
-    void get_fv_eigen_vectors(mdarray<std::complex<T>, 2>& fv_evec__) const;
-
-    /// Test orthonormalization of spinor wave-functions
-    void test_spinor_wave_functions(int use_fft);
+    void get_fv_eigen_vectors(sddk::mdarray<std::complex<T>, 2>& fv_evec__) const;
 
     /// Collect distributed second-variational vectors into a global array.
-    void get_sv_eigen_vectors(mdarray<std::complex<T>, 2>& sv_evec__) const
+    void get_sv_eigen_vectors(sddk::mdarray<std::complex<T>, 2>& sv_evec__) const
     {
         RTE_ASSERT((int)sv_evec__.size(0) == ctx_.num_spins() * ctx_.num_fv_states());
         RTE_ASSERT((int)sv_evec__.size(1) == ctx_.num_spins() * ctx_.num_fv_states());
@@ -393,14 +383,20 @@ class K_point
             }
         }
 
-        comm().allreduce(sv_evec__.at(memory_t::host), (int)sv_evec__.size());
+        comm().allreduce(sv_evec__.at(sddk::memory_t::host), (int)sv_evec__.size());
     }
 
-    inline Gvec const& gkvec() const
+    inline auto const& gkvec() const
     {
         return *gkvec_;
     }
 
+    /// Return shared pointer to gkvec object.
+    inline auto gkvec_sptr() const
+    {
+        return gkvec_;
+    }
+
     /// Total number of G+k vectors within the cutoff distance
     inline int num_gkvec() const
     {
@@ -467,76 +463,60 @@ class K_point
         return weight_;
     }
 
-    inline Wave_functions<T>& fv_states()
+    inline auto& fv_states()
     {
-        assert(fv_states_ != nullptr);
+        RTE_ASSERT(fv_states_ != nullptr);
         return *fv_states_;
     }
 
-    inline Wave_functions<T>& spinor_wave_functions()
+    inline auto const& spinor_wave_functions() const
     {
-        assert(spinor_wave_functions_ != nullptr);
+        RTE_ASSERT(spinor_wave_functions_ != nullptr);
         return *spinor_wave_functions_;
     }
 
-    inline std::shared_ptr<Wave_functions<T>> spinor_wave_functions_ptr()
+    inline auto& spinor_wave_functions()
     {
-        return spinor_wave_functions_;
+        RTE_ASSERT(spinor_wave_functions_ != nullptr);
+        return const_cast<wf::Wave_functions<T>&>(static_cast<K_point const&>(*this).spinor_wave_functions());;
     }
 
-    /// return the initial atomic orbitals used to compute the hubbard wave functions. the S operator is applied on
-    /// these functions
-    inline Wave_functions<T> const& atomic_wave_functions_S() const
+    /// Return the initial atomic orbitals used to compute the hubbard wave functions. The S operator is applied on
+    /// these functions.
+    inline auto const& atomic_wave_functions_S() const
     {
         /* the S operator is applied on these functions */
         RTE_ASSERT(atomic_wave_functions_S_ != nullptr);
         return *atomic_wave_functions_S_;
     }
 
-    inline Wave_functions<T>& atomic_wave_functions_S()
+    inline auto& atomic_wave_functions_S()
     {
-        return const_cast<Wave_functions<T>&>(static_cast<K_point const&>(*this).atomic_wave_functions_S());
+        return const_cast<wf::Wave_functions<T>&>(static_cast<K_point const&>(*this).atomic_wave_functions_S());
     }
 
-    inline Wave_functions<T> const& atomic_wave_functions() const
+    /// Return the initial atomic orbitals used to compute the hubbard wave functions.
+    inline auto const& atomic_wave_functions() const
     {
         RTE_ASSERT(atomic_wave_functions_ != nullptr);
         return *atomic_wave_functions_;
     }
 
-    /// return the initial atomic orbitals used to compute the hubbard wave functions.
-    inline Wave_functions<T>& atomic_wave_functions()
+    /// Return the initial atomic orbitals used to compute the hubbard wave functions.
+    inline auto& atomic_wave_functions()
     {
-        return const_cast<Wave_functions<T>&>(static_cast<K_point const&>(*this).atomic_wave_functions());
+        return const_cast<wf::Wave_functions<T>&>(static_cast<K_point const&>(*this).atomic_wave_functions());
     }
 
-    /// return the actual hubbard wave functions used in the calculations. The S operator is applied when uspp are used.
-    inline Wave_functions<T> const& hubbard_wave_functions_S() const
+    /// Return the actual hubbard wave functions used in the calculations.
+    /** The S operator is applied on these functions. */
+    inline auto const& hubbard_wave_functions_S() const
     {
-        assert(hubbard_wave_functions_S_ != nullptr);
+        RTE_ASSERT(hubbard_wave_functions_S_ != nullptr);
         return *hubbard_wave_functions_S_;
     }
 
-    /// return the actual hubbard wave functions used in the calculations. The S operator is applied when uspp are used.
-    inline Wave_functions<T>& hubbard_wave_functions_S()
-    {
-        return const_cast<Wave_functions<T>&>(static_cast<K_point const&>(*this).hubbard_wave_functions_S());
-    }
-
-    /// return the actual hubbard wave functions used in the calculations.
-    inline Wave_functions<T> const& hubbard_wave_functions() const
-    {
-        assert(hubbard_wave_functions_ != nullptr);
-        return *hubbard_wave_functions_;
-    }
-
-    /// return the actual hubbard wave functions used in the calculations.
-    inline Wave_functions<T>& hubbard_wave_functions()
-    {
-        return const_cast<Wave_functions<T>&>(static_cast<K_point const&>(*this).hubbard_wave_functions());
-    }
-
-    inline Wave_functions<T>& singular_components()
+    inline auto& singular_components()
     {
         return *singular_components_;
     }
@@ -554,12 +534,6 @@ class K_point
         return num_gkvec() + unit_cell_.mt_lo_basis_size();
     }
 
-    /// Return global index of G+k vector. // TODO: consider removal
-    inline int idxgk(int igkloc__) const
-    {
-        return gkvec_->offset() + igkloc__;
-    }
-
     /// Local number of G+k vectors for each MPI rank in the row of the 2D MPI grid.
     inline int num_gkvec_row() const
     {
@@ -598,13 +572,13 @@ class K_point
 
     inline lo_basis_descriptor const& lo_basis_descriptor_col(int idx) const
     {
-        assert(idx >= 0 && idx < (int)lo_basis_descriptors_col_.size());
+        RTE_ASSERT(idx >= 0 && idx < (int)lo_basis_descriptors_col_.size());
         return lo_basis_descriptors_col_[idx];
     }
 
     inline lo_basis_descriptor const& lo_basis_descriptor_row(int idx) const
     {
-        assert(idx >= 0 && idx < (int)lo_basis_descriptors_row_.size());
+        RTE_ASSERT(idx >= 0 && idx < (int)lo_basis_descriptors_row_.size());
         return lo_basis_descriptors_row_[idx];
     }
 
@@ -618,7 +592,7 @@ class K_point
         return igk_loc_;
     }
 
-    inline int igk_row(int idx__) const
+    inline int igk_row(int idx__) const // TODO: get all from gkvec_row_
     {
         return igk_row_[idx__];
     }
@@ -693,22 +667,22 @@ class K_point
         return atom_lo_rows_[ia][i];
     }
 
-    inline dmatrix<std::complex<T>>& fv_eigen_vectors()
+    inline auto& fv_eigen_vectors()
     {
         return fv_eigen_vectors_;
     }
 
-    inline Wave_functions<T>& fv_eigen_vectors_slab()
+    inline auto& fv_eigen_vectors_slab()
     {
         return *fv_eigen_vectors_slab_;
     }
 
-    inline dmatrix<std::complex<T>>& sv_eigen_vectors(int ispn)
+    inline auto& sv_eigen_vectors(int ispn)
     {
         return sv_eigen_vectors_[ispn];
     }
 
-    inline mdarray<std::complex<T>, 2>& fd_eigen_vectors()
+    inline auto& fd_eigen_vectors()
     {
         return fd_eigen_vectors_;
     }
@@ -718,103 +692,127 @@ class K_point
         std::copy(&fv_eigen_values_[0], &fv_eigen_values_[0] + ctx_.num_fv_states(), &band_energies_[0]);
     }
 
-    inline Matching_coefficients const& alm_coeffs_row() const
+    inline auto const& alm_coeffs_row() const
     {
         return *alm_coeffs_row_;
     }
 
-    inline Matching_coefficients const& alm_coeffs_col() const
+    inline auto const& alm_coeffs_col() const
     {
         return *alm_coeffs_col_;
     }
 
-    inline Matching_coefficients const& alm_coeffs_loc() const
+    inline auto const& alm_coeffs_loc() const
     {
         return *alm_coeffs_loc_;
     }
 
-    inline Communicator const& comm() const
+    inline auto const& comm() const
     {
         return comm_;
     }
 
-    inline Communicator const& comm_row() const
+    inline auto const& comm_row() const
     {
         return comm_row_;
     }
 
-    inline Communicator const& comm_col() const
+    inline auto const& comm_col() const
     {
         return comm_col_;
     }
 
-    inline std::complex<T> p_mtrx(int xi1, int xi2, int iat) const
+    inline auto p_mtrx(int xi1, int xi2, int iat) const
     {
         return p_mtrx_(xi1, xi2, iat);
     }
 
-    inline mdarray<std::complex<T>, 3>& p_mtrx()
+    inline auto& p_mtrx()
     {
         return p_mtrx_;
     }
 
-    Beta_projectors<T>& beta_projectors()
+    auto& beta_projectors()
     {
-        assert(beta_projectors_ != nullptr);
+        RTE_ASSERT(beta_projectors_ != nullptr);
         return *beta_projectors_;
     }
 
-    Beta_projectors<T> const& beta_projectors() const
+    auto const& beta_projectors() const
     {
-        assert(beta_projectors_ != nullptr);
+        RTE_ASSERT(beta_projectors_ != nullptr);
         return *beta_projectors_;
     }
 
-    Beta_projectors<T>& beta_projectors_row()
+    auto& beta_projectors_row()
     {
-        assert(beta_projectors_ != nullptr);
+        RTE_ASSERT(beta_projectors_ != nullptr);
         return *beta_projectors_row_;
     }
 
-    Beta_projectors<T>& beta_projectors_col()
+    auto& beta_projectors_col()
     {
-        assert(beta_projectors_ != nullptr);
+        RTE_ASSERT(beta_projectors_ != nullptr);
         return *beta_projectors_col_;
     }
 
-    Simulation_context const& ctx() const
+    auto const& ctx() const
     {
         return ctx_;
     }
 
-    /// Print message from the root rank.
-    template <typename... Args>
-    inline void message(int level__, char const* label__, Args... args) const
+    /// Return stdout stream for high verbosity output.
+    /** This output will not be passed to a ctx_.out() stream. */
+    std::ostream& out(int level__) const
     {
-        if (this->comm().rank() == 0 && this->ctx().cfg().control().verbosity() >= level__) {
-            if (label__) {
-                std::printf("[%s] ", label__);
-            }
-            std::printf(args...);
+        if (ctx_.verbosity() >= level__ && this->comm().rank() == 0) {
+            return std::cout;
+        } else {
+            return utils::null_stream();
         }
     }
 
-    spfft_transform_type<T>& spfft_transform()
+    auto& spfft_transform()
     {
         return *spfft_transform_;
     }
 
-    spfft_transform_type<T> const& spfft_transform() const
+    auto const& spfft_transform() const
     {
         return *spfft_transform_;
     }
 
-    inline Gvec_partition const& gkvec_partition() const
+    inline auto const& gkvec_fft() const
     {
         return *gkvec_partition_;
     }
+
+    inline auto gkvec_fft_sptr() const
+    {
+        return gkvec_partition_;
+    }
 };
 
+template <typename T>
+inline auto
+wave_function_factory(Simulation_context const& ctx__, K_point<T> const& kp__, wf::num_bands num_wf__,
+        wf::num_mag_dims num_md__, bool mt_part__)
+{
+    using wf_t = wf::Wave_functions<T>;
+    std::unique_ptr<wf_t> wf{nullptr};
+    if (mt_part__) {
+        std::vector<int> num_mt_coeffs(ctx__.unit_cell().num_atoms());
+        for (int ia = 0; ia < ctx__.unit_cell().num_atoms(); ia++) {
+            num_mt_coeffs[ia] = ctx__.unit_cell().atom(ia).mt_lo_basis_size();
+        }
+        wf = std::make_unique<wf_t>(kp__.gkvec_sptr(), num_mt_coeffs, num_md__, num_wf__, ctx__.host_memory_t());
+    } else {
+        wf = std::make_unique<wf_t>(kp__.gkvec_sptr(), num_md__, num_wf__, ctx__.host_memory_t());
+    }
+
+    return wf;
+}
+
 } // namespace sirius
 
 /** \page basis Basis functions for Kohn-Sham wave-functions expansion
