@@ -31,8 +31,8 @@ void K_point_set::sync_band()
 {
     PROFILE("sirius::K_point_set::sync_band");
 
-    sddk::mdarray<double, 3> data(ctx_.num_bands(), ctx_.num_spinors(), num_kpoints(), ctx_.mem_pool(memory_t::host),
-                                  "K_point_set::sync_band.data");
+    sddk::mdarray<double, 3> data(ctx_.num_bands(), ctx_.num_spinors(), num_kpoints(),
+            get_memory_pool(sddk::memory_t::host), "K_point_set::sync_band.data");
 
     int nb = ctx_.num_bands() * ctx_.num_spinors();
     #pragma omp parallel
@@ -51,7 +51,7 @@ void K_point_set::sync_band()
         }
     }
 
-    comm().allgather(data.at(memory_t::host), nb * spl_num_kpoints_.local_size(),
+    comm().allgather(data.at(sddk::memory_t::host), nb * spl_num_kpoints_.local_size(),
         nb * spl_num_kpoints_.global_offset());
 
     #pragma omp parallel for
@@ -93,14 +93,14 @@ void K_point_set::create_k_mesh(vector3d<int> k_grid__, vector3d<int> k_shift__,
     PROFILE("sirius::K_point_set::create_k_mesh");
 
     int nk;
-    mdarray<double, 2> kp;
+    sddk::mdarray<double, 2> kp;
     std::vector<double> wk;
     if (use_symmetry__) {
         auto result = get_irreducible_reciprocal_mesh(ctx_.unit_cell().symmetry(), k_grid__, k_shift__);
         nk          = std::get<0>(result);
         wk          = std::get<1>(result);
         auto tmp    = std::get<2>(result);
-        kp          = mdarray<double, 2>(3, nk);
+        kp          = sddk::mdarray<double, 2>(3, nk);
         for (int i = 0; i < nk; i++) {
             for (int x : {0, 1, 2}) {
                 kp(x, i) = tmp[i][x];
@@ -109,7 +109,7 @@ void K_point_set::create_k_mesh(vector3d<int> k_grid__, vector3d<int> k_shift__,
     } else {
         nk = k_grid__[0] * k_grid__[1] * k_grid__[2];
         wk = std::vector<double>(nk, 1.0 / nk);
-        kp = mdarray<double, 2>(3, nk);
+        kp = sddk::mdarray<double, 2>(3, nk);
 
         int ik = 0;
         for (int i0 = 0; i0 < k_grid__[0]; i0++) {
@@ -139,10 +139,10 @@ void K_point_set::initialize(std::vector<int> const& counts)
     PROFILE("sirius::K_point_set::initialize");
     /* distribute k-points along the 1-st dimension of the MPI grid */
     if (counts.empty()) {
-        splindex<splindex_t::block> spl_tmp(num_kpoints(), comm().size(), comm().rank());
-        spl_num_kpoints_ = splindex<splindex_t::chunk>(num_kpoints(), comm().size(), comm().rank(), spl_tmp.counts());
+        sddk::splindex<sddk::splindex_t::block> spl_tmp(num_kpoints(), comm().size(), comm().rank());
+        spl_num_kpoints_ = sddk::splindex<sddk::splindex_t::chunk>(num_kpoints(), comm().size(), comm().rank(), spl_tmp.counts());
     } else {
-        spl_num_kpoints_ = splindex<splindex_t::chunk>(num_kpoints(), comm().size(), comm().rank(), counts);
+        spl_num_kpoints_ = sddk::splindex<sddk::splindex_t::chunk>(num_kpoints(), comm().size(), comm().rank(), counts);
     }
 
     for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
@@ -155,7 +155,7 @@ void K_point_set::initialize(std::vector<int> const& counts)
     if (ctx_.verbosity() > 0) {
         this->print_info();
     }
-    ctx_.print_memory_usage(__FILE__, __LINE__);
+    print_memory_usage(ctx_.out(), FILE_LINE);
     this->initialized_ = true;
 }
 
@@ -314,7 +314,7 @@ void K_point_set::find_band_occupancies()
     comm().allreduce<double, sddk::mpi_op_t::min>(&emin, 1);
     comm().allreduce<double, sddk::mpi_op_t::max>(&emax, 1);
 
-    splindex<splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
+    sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
 
     /* computes N(ef; f) = \sum_{i,k} f(ef - e_{k,i}) */
     auto compute_ne = [&](double ef, auto&& f) {
@@ -358,12 +358,15 @@ void K_point_set::find_band_occupancies()
             auto ddN = [&](double mu) { return compute_ne(mu, ddf); };
             auto res_newton =  newton_minimization_chemical_potential(N, dN, ddN, energy_fermi_, ne_target, tol, 300);
             energy_fermi_ = res_newton.mu;
-            ctx_.message(2, __function_name__, "newton iteration converged after %d steps\n", res_newton.iter);
-
+            if (ctx_.verbosity() >= 2) {
+                RTE_OUT(ctx_.out()) << "newton iteration converged after " << res_newton.iter << " steps\n";
+            }
         }
     } catch(std::exception const& e) {
-        ctx_.message(2, __function_name__, "%s\n", e.what());
-        ctx_.message(2, __function_name__, "%s\n", "fallback to bisection search\n");
+        if (ctx_.verbosity() >= 2) {
+            RTE_OUT(ctx_.out()) << e.what() << std::endl
+                << "fallback to bisection search" << std::endl;
+        }
         f             = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
         auto F        = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
         energy_fermi_ = bisection_search(F, emin, emax, tol);
@@ -430,7 +433,7 @@ double K_point_set::valence_eval_sum() const
 {
     double eval_sum{0};
 
-    splindex<splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
+    sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
 
     for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
         auto ik = spl_num_kpoints_[ikloc];
@@ -479,7 +482,7 @@ double K_point_set::entropy_sum() const
 
     auto f = smearing::entropy(ctx_.smearing(), ctx_.smearing_width());
 
-    splindex<splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
+    sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
 
     for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
         auto ik = spl_num_kpoints_[ikloc];
@@ -514,47 +517,41 @@ double K_point_set::entropy_sum() const
 
 void K_point_set::print_info()
 {
+    sddk::pstdout pout(this->comm());
+
     if (ctx_.comm().rank() == 0) {
-        std::printf("\n");
-        std::printf("total number of k-points : %i\n", num_kpoints());
-        for (int i = 0; i < 80; i++) {
-            std::printf("-");
-        }
-        std::printf("\n");
-        std::printf("  ik                vk                    weight  num_gkvec");
+        pout << std::endl;
+        pout << "total number of k-points : " << num_kpoints() << std::endl;
+        pout << utils::hbar(80, '-') << std::endl;
+        pout << std::endl;
+        pout << "  ik                vk                    weight  num_gkvec";
         if (ctx_.full_potential()) {
-            std::printf("   gklo_basis_size");
+            pout << "   gklo_basis_size";
         }
-        std::printf("\n");
-        for (int i = 0; i < 80; i++) {
-            std::printf("-");
-        }
-        std::printf("\n");
+        pout << std::endl << utils::hbar(80, '-') << std::endl;
     }
 
-    if (ctx_.comm_band().rank() == 0) {
-        pstdout pout(comm());
-        for (int ikloc = 0; ikloc < spl_num_kpoints().local_size(); ikloc++) {
-            int ik = spl_num_kpoints(ikloc);
-            pout.printf("%4i   %8.4f %8.4f %8.4f   %12.6f     %6i", ik, kpoints_[ik]->vk()[0], kpoints_[ik]->vk()[1],
-                        kpoints_[ik]->vk()[2], kpoints_[ik]->weight(), kpoints_[ik]->num_gkvec());
+    for (int ikloc = 0; ikloc < spl_num_kpoints().local_size(); ikloc++) {
+        int ik = spl_num_kpoints(ikloc);
+        pout << std::setw(4) << ik << utils::ffmt(9, 4) << kpoints_[ik]->vk()[0] << utils::ffmt(9, 4)
+             << kpoints_[ik]->vk()[1] << utils::ffmt(9, 4) << kpoints_[ik]->vk()[2] << utils::ffmt(17, 6)
+             << kpoints_[ik]->weight() << std::setw(11) << kpoints_[ik]->num_gkvec();
 
-            if (ctx_.full_potential()) {
-                pout.printf("            %6i", kpoints_[ik]->gklo_basis_size());
-            }
-
-            pout.printf("\n");
+        if (ctx_.full_potential()) {
+            pout << std::setw(18) << kpoints_[ik]->gklo_basis_size();
         }
+        pout << std::endl;
     }
+    RTE_OUT(ctx_.out()) << pout.flush(0);
 }
 
 void K_point_set::save(std::string const& name__) const
 {
     if (ctx_.comm().rank() == 0) {
         if (!utils::file_exists(name__)) {
-            HDF5_tree(name__, hdf5_access_t::truncate);
+            sddk::HDF5_tree(name__, sddk::hdf5_access_t::truncate);
         }
-        HDF5_tree fout(name__, hdf5_access_t::read_write);
+        sddk::HDF5_tree fout(name__, sddk::hdf5_access_t::read_write);
         fout.create_node("K_point_set");
         fout["K_point_set"].write("num_kpoints", num_kpoints());
     }
