@@ -57,7 +57,7 @@ class Matching_coefficients // TODO: compute on GPU
     Unit_cell const& unit_cell_;
 
     /// Description of the G+k vectors.
-    Gvec const& gkvec_;
+    sddk::Gvec const& gkvec_;
 
     std::vector<double> gkvec_len_;
 
@@ -109,7 +109,7 @@ class Matching_coefficients // TODO: compute on GPU
 
   public:
     /// Constructor
-    Matching_coefficients(Unit_cell const& unit_cell__, Gvec const& gkvec__)
+    Matching_coefficients(Unit_cell const& unit_cell__, sddk::Gvec const& gkvec__)
         : unit_cell_(unit_cell__)
         , gkvec_(gkvec__)
     {
@@ -126,7 +126,7 @@ class Matching_coefficients // TODO: compute on GPU
 
             #pragma omp for
             for (int i = 0; i < gkvec_.count(); i++) {
-                auto gkvec_cart = gkvec_.gkvec_cart<index_domain_t::local>(i);
+                auto gkvec_cart = gkvec_.gkvec_cart<sddk::index_domain_t::local>(i);
                 /* get r, theta, phi */
                 auto vs = SHT::spherical_coordinates(gkvec_cart);
                 gkvec_len_[i] = vs[0];
@@ -200,7 +200,7 @@ class Matching_coefficients // TODO: compute on GPU
 
         std::vector<double_complex> phase_factors(gkvec_.count());
         for (int i = 0; i < gkvec_.count(); i++) {
-            double phase     = twopi * dot(gkvec_.template gkvec<index_domain_t::local>(i), atom__.position());
+            double phase     = twopi * dot(gkvec_.template gkvec<sddk::index_domain_t::local>(i), atom__.position());
             phase_factors[i] = std::exp(double_complex(0, phase));
         }
 
@@ -292,6 +292,11 @@ class Matching_coefficients // TODO: compute on GPU
             }
         }
     }
+
+    auto const& gkvec() const
+    {
+        return gkvec_;
+    }
 };
 
 } // namespace sirius
