@@ -39,7 +39,7 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
 
     bool const use_2nd_deriv{false};
 
-    auto& gvp = ctx_.gvec_partition();
+    auto gvp = ctx_.gvec_fft_sptr();
 
     bool is_gga = is_gradient_correction();
 
@@ -66,7 +66,7 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
         rhomin = std::min(rhomin, d);
         rho.f_rg(ir) = std::max(d, 0.0);
     }
-    Communicator(ctx_.spfft<double>().communicator()).allreduce<double, mpi_op_t::min>(&rhomin, 1);
+    sddk::Communicator(ctx_.spfft<double>().communicator()).allreduce<double, sddk::mpi_op_t::min>(&rhomin, 1);
     /* even a small negative density is a sign of something bing wrong; don't remove this check */
     if (rhomin < 0.0 && ctx_.comm().rank() == 0) {
         std::stringstream s;
@@ -84,9 +84,7 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
 
     if (ctx_.cfg().control().print_checksum()) {
         auto cs = density__.rho().checksum_rg();
-        if (ctx_.comm().rank() == 0) {
-            utils::print_checksum("rho_rg", cs);
-        }
+        utils::print_checksum("rho_rg", cs, ctx_.out());
     }
 
     Smooth_periodic_vector_function<double> grad_rho;
@@ -128,12 +126,12 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
 
     Smooth_periodic_function<double> vsigma;
     if (is_gga) {
-        vsigma = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_partition());
+        vsigma = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
         vsigma_[0]->zero();
     }
 
-    sddk::mdarray<double, 1> exc(num_points, memory_t::host, "exc_tmp");
-    sddk::mdarray<double, 1> vxc(num_points, memory_t::host, "vxc_tmp");
+    sddk::mdarray<double, 1> exc(num_points, sddk::memory_t::host, "exc_tmp");
+    sddk::mdarray<double, 1> vxc(num_points, sddk::memory_t::host, "vxc_tmp");
 
     /* loop over XC functionals */
     for (auto& ixc: xc_func_) {
@@ -143,8 +141,8 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
             /* all ranks should make a call because VdW uses FFT internaly */
             if (num_points) {
                 /* Van der Walls correction */
-                ixc.get_vdw(&rho.f_rg(0), &grad_rho_grad_rho.f_rg(0), vxc.at(memory_t::host), &vsigma.f_rg(0),
-                             exc.at(memory_t::host));
+                ixc.get_vdw(&rho.f_rg(0), &grad_rho_grad_rho.f_rg(0), vxc.at(sddk::memory_t::host), &vsigma.f_rg(0),
+                             exc.at(sddk::memory_t::host));
             } else {
                 ixc.get_vdw(nullptr, nullptr, nullptr, nullptr, nullptr);
             }
@@ -156,20 +154,20 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
             #pragma omp parallel
             {
                 /* split local size between threads */
-                splindex<splindex_t::block> spl_t(num_points, omp_get_num_threads(), omp_get_thread_num());
+                sddk::splindex<sddk::splindex_t::block> spl_t(num_points, omp_get_num_threads(), omp_get_thread_num());
                 /* if this is an LDA functional */
                 if (ixc.is_lda()) {
                     ixc.get_lda(spl_t.local_size(), &rho.f_rg(spl_t.global_offset()),
-                                vxc.at(memory_t::host, spl_t.global_offset()),
-                                exc.at(memory_t::host, spl_t.global_offset()));
+                                vxc.at(sddk::memory_t::host, spl_t.global_offset()),
+                                exc.at(sddk::memory_t::host, spl_t.global_offset()));
                 }
                 /* if this is a GGA functional */
                 if (ixc.is_gga()) {
                     ixc.get_gga(spl_t.local_size(), &rho.f_rg(spl_t.global_offset()),
                                 &grad_rho_grad_rho.f_rg(spl_t.global_offset()),
-                                vxc.at(memory_t::host, spl_t.global_offset()),
+                                vxc.at(sddk::memory_t::host, spl_t.global_offset()),
                                 &vsigma.f_rg(spl_t.global_offset()),
-                                exc.at(memory_t::host, spl_t.global_offset()));
+                                exc.at(sddk::memory_t::host, spl_t.global_offset()));
                 }
             } // omp parallel region
                         ///* this is the same expression between gga and vdw corrections.
@@ -245,9 +243,7 @@ void Potential::xc_rg_nonmagnetic(Density const& density__)
 
     if (ctx_.cfg().control().print_checksum()) {
         auto cs = xc_potential_->checksum_rg();
-        if (ctx_.comm().rank() == 0) {
-            utils::print_checksum("exc", cs);
-        }
+        utils::print_checksum("exc", cs, ctx_.out());
     }
 }
 
@@ -322,17 +318,17 @@ void Potential::xc_rg_magnetic(Density const& density__)
     Smooth_periodic_function<double> vsigma_dd;
 
     if (is_gga) {
-        vsigma_uu = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_partition());
-        vsigma_ud = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_partition());
-        vsigma_dd = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_partition());
+        vsigma_uu = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
+        vsigma_ud = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
+        vsigma_dd = Smooth_periodic_function<double>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
         for (int i = 0; i < 3; i++) {
             vsigma_[i]->zero();
         }
     }
 
-    sddk::mdarray<double, 1> exc(num_points, memory_t::host, "exc_tmp");
-    sddk::mdarray<double, 1> vxc_up(num_points, memory_t::host, "vxc_up_tmp");
-    sddk::mdarray<double, 1> vxc_dn(num_points, memory_t::host, "vxc_dn_dmp");
+    sddk::mdarray<double, 1> exc(num_points, sddk::memory_t::host, "exc_tmp");
+    sddk::mdarray<double, 1> vxc_up(num_points, sddk::memory_t::host, "vxc_up_tmp");
+    sddk::mdarray<double, 1> vxc_dn(num_points, sddk::memory_t::host, "vxc_dn_dmp");
 
     /* loop over XC functionals */
     for (auto& ixc: xc_func_) {
@@ -342,8 +338,8 @@ void Potential::xc_rg_magnetic(Density const& density__)
             /* all ranks should make a call because VdW uses FFT internaly */
             if (num_points) {
                 ixc.get_vdw(&rho_up.f_rg(0), &rho_dn.f_rg(0), &grad_rho_up_grad_rho_up.f_rg(0),
-                             &grad_rho_dn_grad_rho_dn.f_rg(0), vxc_up.at(memory_t::host), vxc_dn.at(memory_t::host),
-                             &vsigma_uu.f_rg(0), &vsigma_dd.f_rg(0), exc.at(memory_t::host));
+                             &grad_rho_dn_grad_rho_dn.f_rg(0), vxc_up.at(sddk::memory_t::host), vxc_dn.at(sddk::memory_t::host),
+                             &vsigma_uu.f_rg(0), &vsigma_dd.f_rg(0), exc.at(sddk::memory_t::host));
             } else {
                 ixc.get_vdw(nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr);
             }
@@ -355,14 +351,14 @@ void Potential::xc_rg_magnetic(Density const& density__)
             #pragma omp parallel
             {
                 /* split local size between threads */
-                splindex<splindex_t::block> spl_t(num_points, omp_get_num_threads(), omp_get_thread_num());
+                sddk::splindex<sddk::splindex_t::block> spl_t(num_points, omp_get_num_threads(), omp_get_thread_num());
                 /* if this is an LDA functional */
                 if (ixc.is_lda()) {
                     ixc.get_lda(spl_t.local_size(), &rho_up.f_rg(spl_t.global_offset()),
                                 &rho_dn.f_rg(spl_t.global_offset()),
-                                vxc_up.at(memory_t::host, spl_t.global_offset()),
-                                vxc_dn.at(memory_t::host, spl_t.global_offset()),
-                                exc.at(memory_t::host, spl_t.global_offset()));
+                                vxc_up.at(sddk::memory_t::host, spl_t.global_offset()),
+                                vxc_dn.at(sddk::memory_t::host, spl_t.global_offset()),
+                                exc.at(sddk::memory_t::host, spl_t.global_offset()));
                 }
                 /* if this is a GGA functional */
                 if (ixc.is_gga()) {
@@ -371,12 +367,12 @@ void Potential::xc_rg_magnetic(Density const& density__)
                                 &grad_rho_up_grad_rho_up.f_rg(spl_t.global_offset()),
                                 &grad_rho_up_grad_rho_dn.f_rg(spl_t.global_offset()),
                                 &grad_rho_dn_grad_rho_dn.f_rg(spl_t.global_offset()),
-                                vxc_up.at(memory_t::host, spl_t.global_offset()),
-                                vxc_dn.at(memory_t::host, spl_t.global_offset()),
+                                vxc_up.at(sddk::memory_t::host, spl_t.global_offset()),
+                                vxc_dn.at(sddk::memory_t::host, spl_t.global_offset()),
                                 &vsigma_uu.f_rg(spl_t.global_offset()),
                                 &vsigma_ud.f_rg(spl_t.global_offset()),
                                 &vsigma_dd.f_rg(spl_t.global_offset()),
-                                exc.at(memory_t::host, spl_t.global_offset()));
+                                exc.at(sddk::memory_t::host, spl_t.global_offset()));
                 }
             } // omp parallel region
             } // num_points != 0
@@ -391,8 +387,8 @@ void Potential::xc_rg_magnetic(Density const& density__)
                 vsigma_[2]->f_rg(ir) += vsigma_dd.f_rg(ir);
             }
 
-            Smooth_periodic_vector_function<double> up_gradrho_vsigma(ctx_.spfft<double>(), ctx_.gvec_partition());
-            Smooth_periodic_vector_function<double> dn_gradrho_vsigma(ctx_.spfft<double>(), ctx_.gvec_partition());
+            Smooth_periodic_vector_function<double> up_gradrho_vsigma(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
+            Smooth_periodic_vector_function<double> dn_gradrho_vsigma(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
             for (int x: {0, 1, 2}) {
                 for(int ir = 0; ir < num_points; ir++) {
                   up_gradrho_vsigma[x].f_rg(ir) = 2 * grad_rho_up[x].f_rg(ir) * vsigma_uu.f_rg(ir) + grad_rho_dn[x].f_rg(ir) * vsigma_ud.f_rg(ir);
