@@ -150,24 +150,24 @@ Atom_type::init(int offset_lo__)
                 }
             }
         }
-        idx_radial_integrals_ = mdarray<int, 2>(2, non_zero_elements.size());
+        idx_radial_integrals_ = sddk::mdarray<int, 2>(2, non_zero_elements.size());
         for (int j = 0; j < (int)non_zero_elements.size(); j++) {
             idx_radial_integrals_(0, j) = non_zero_elements[j].first;
             idx_radial_integrals_(1, j) = non_zero_elements[j].second;
         }
     }
 
-    if (parameters_.processing_unit() == device_t::GPU && parameters_.full_potential()) {
-        idx_radial_integrals_.allocate(memory_t::device).copy_to(memory_t::device);
+    if (parameters_.processing_unit() == sddk::device_t::GPU && parameters_.full_potential()) {
+        idx_radial_integrals_.allocate(sddk::memory_t::device).copy_to(sddk::memory_t::device);
         rf_coef_ =
-            mdarray<double, 3>(num_mt_points(), 4, indexr().size(), memory_t::host_pinned, "Atom_type::rf_coef_");
+            sddk::mdarray<double, 3>(num_mt_points(), 4, indexr().size(), sddk::memory_t::host_pinned, "Atom_type::rf_coef_");
         vrf_coef_ =
-            mdarray<double, 3>(num_mt_points(), 4, lmmax_pot * indexr().size() * (parameters_.num_mag_dims() + 1),
-                               memory_t::host_pinned, "Atom_type::vrf_coef_");
-        rf_coef_.allocate(memory_t::device);
-        vrf_coef_.allocate(memory_t::device);
+            sddk::mdarray<double, 3>(num_mt_points(), 4, lmmax_pot * indexr().size() * (parameters_.num_mag_dims() + 1),
+                               sddk::memory_t::host_pinned, "Atom_type::vrf_coef_");
+        rf_coef_.allocate(sddk::memory_t::device);
+        vrf_coef_.allocate(sddk::memory_t::device);
     }
-    if (parameters_.processing_unit() == device_t::GPU) {
+    if (parameters_.processing_unit() == sddk::device_t::GPU) {
         radial_grid_.copy_to_device();
     }
 
@@ -182,9 +182,9 @@ Atom_type::init(int offset_lo__)
         if (num_beta_radial_functions() != num_ae_paw_wf()) {
             RTE_THROW("wrong number of all-electron wave-functions for PAW");
         }
-        ae_paw_wfs_array_ = mdarray<double, 2>(num_mt_points(), num_beta_radial_functions());
+        ae_paw_wfs_array_ = sddk::mdarray<double, 2>(num_mt_points(), num_beta_radial_functions());
         ae_paw_wfs_array_.zero();
-        ps_paw_wfs_array_ = mdarray<double, 2>(num_mt_points(), num_beta_radial_functions());
+        ps_paw_wfs_array_ = sddk::mdarray<double, 2>(num_mt_points(), num_beta_radial_functions());
         ps_paw_wfs_array_.zero();
 
         for (int i = 0; i < num_beta_radial_functions(); i++) {
@@ -251,23 +251,18 @@ Atom_type::init_free_atom_density(bool smooth)
 void
 Atom_type::print_info(std::ostream& out__) const
 {
-    auto draw_bar = [&](char c, int w)
-    {
-        out__ << std::setfill(c) << std::setw(w) << c << std::setfill(' ') << std::endl;
-    };
-
-    out__ << "label          : " << label() << std::endl;
-    draw_bar('-', 80);
-    out__ << "symbol         : " << symbol_ << std::endl
+    out__ << "label          : " << label() << std::endl
+          << utils::hbar(80, '-') << std::endl
+          << "symbol         : " << symbol_ << std::endl
           << "name           : " << name_ << std::endl
           << "zn             : " << zn_ << std::endl
           << "mass           : " << mass_ << std::endl
           << "mt_radius      : " << mt_radius() << std::endl
           << "num_mt_points  : " << num_mt_points() << std::endl
           << "grid_origin    : " << radial_grid_.first() << std::endl
-          << "grid_name      : " << radial_grid_.name() << std::endl;
-    out__ << std::endl;
-    out__ << "number of core electrons    : " << num_core_electrons_ << std::endl
+          << "grid_name      : " << radial_grid_.name() << std::endl
+          << std::endl
+          << "number of core electrons    : " << num_core_electrons_ << std::endl
           << "number of valence electrons : " << num_valence_electrons_ << std::endl;
 
     if (parameters_.full_potential()) {
@@ -355,6 +350,7 @@ Atom_type::print_info(std::ostream& out__) const
         out__ << indexr_wfs_.am(i);
     }
     out__ << std::endl;
+    out__ << std::endl;
 }
 
 void
@@ -539,7 +535,7 @@ Atom_type::read_pseudo_uspp(nlohmann::json const& parser)
         add_beta_radial_function(l, beta);
     }
 
-    mdarray<double, 2> d_mtrx(nbf, nbf);
+    sddk::mdarray<double, 2> d_mtrx(nbf, nbf);
     d_mtrx.zero();
     auto v = parser["pseudo_potential"]["D_ion"].get<std::vector<double>>();
 
@@ -732,7 +728,7 @@ Atom_type::generate_f_coefficients()
 
     // number of beta projectors
     int nbf         = this->mt_basis_size();
-    f_coefficients_ = mdarray<double_complex, 4>(nbf, nbf, 2, 2);
+    f_coefficients_ = sddk::mdarray<double_complex, 4>(nbf, nbf, 2, 2);
     f_coefficients_.zero();
 
     for (int xi2 = 0; xi2 < nbf; xi2++) {
